@@ -135,7 +135,6 @@ EXCLUDE = (
     "lib/ast/pathcat.c",
     "lib/ast/pathfind.c",
     "lib/ast/pathpath.c",
-    "lib/ast/sfstr.h",
     "lib/ast/strcopy.c",
     "lib/ast/stresc.c",
     "lib/ast/strmatch.c",
@@ -163,7 +162,6 @@ EXCLUDE = (
     "lib/cgraph/cghdr.h",
     "lib/cgraph/cgraph.h",
     "lib/cgraph/edge.c",
-    "lib/cgraph/flatten.c",
     "lib/cgraph/graph.c",
     "lib/cgraph/id.c",
     "lib/cgraph/imap.c",
@@ -171,7 +169,6 @@ EXCLUDE = (
     "lib/cgraph/mem.c",
     "lib/cgraph/node.c",
     "lib/cgraph/obj.c",
-    "lib/cgraph/pend.c",
     "lib/cgraph/rec.c",
     "lib/cgraph/refstr.c",
     "lib/cgraph/subg.c",
@@ -279,7 +276,6 @@ EXCLUDE = (
     "lib/expr/exnospace.c",
     "lib/expr/exopen.c",
     "lib/expr/expr.h",
-    "lib/expr/exstash.c",
     "lib/expr/extoken.c",
     "lib/expr/extype.c",
     "lib/expr/exzero.c",
@@ -514,43 +510,15 @@ EXCLUDE = (
     "lib/sfdpgen/spring_electrical.c",
     "lib/sfdpgen/spring_electrical.h",
     "lib/sfdpgen/stress_model.c",
-    "lib/sfdpgen/stress_model.h",
-    "lib/sfio/Sfio_f/_sffileno.c",
-    "lib/sfio/Sfio_f/_sfgetc.c",
-    "lib/sfio/Sfio_f/_sfputc.c",
     "lib/sfio/Sfio_f/_sfslen.c",
-    "lib/sfio/sfclose.c",
     "lib/sfio/sfcvt.c",
-    "lib/sfio/sfdisc.c",
-    "lib/sfio/sfexcept.c",
     "lib/sfio/sfextern.c",
-    "lib/sfio/sffilbuf.c",
-    "lib/sfio/sfflsbuf.c",
     "lib/sfio/sfhdr.h",
     "lib/sfio/sfio.h",
-    "lib/sfio/sfio_t.h",
-    "lib/sfio/sfmode.c",
-    "lib/sfio/sfnew.c",
-    "lib/sfio/sfnputc.c",
-    "lib/sfio/sfopen.c",
-    "lib/sfio/sfpkrd.c",
-    "lib/sfio/sfputr.c",
-    "lib/sfio/sfraise.c",
-    "lib/sfio/sfrd.c",
-    "lib/sfio/sfread.c",
     "lib/sfio/sfscanf.c",
-    "lib/sfio/sfsetbuf.c",
-    "lib/sfio/sfsetfd.c",
-    "lib/sfio/sfsk.c",
-    "lib/sfio/sfstack.c",
-    "lib/sfio/sfswap.c",
-    "lib/sfio/sfsync.c",
     "lib/sfio/sftable.c",
-    "lib/sfio/sfungetc.c",
     "lib/sfio/sfprint.c",
     "lib/sfio/sfvscanf.c",
-    "lib/sfio/sfwr.c",
-    "lib/sfio/sfwrite.c",
     "lib/sparse/BinaryHeap.c",
     "lib/sparse/BinaryHeap.h",
     "lib/sparse/DotIO.c",
@@ -716,11 +684,6 @@ EXCLUDE = (
     "tclpkg/tclpathplan/wrapper.c",
     "tclpkg/tclstubs/tclInt.h",
     "tclpkg/tclstubs/tclStubLib.c",
-    "tclpkg/tkspline/dllEntry.c",
-    "tclpkg/tkspline/tkspline.c",
-    "tclpkg/tkstubs/tkInt.h",
-    "tclpkg/tkstubs/tkStubImg.c",
-    "tclpkg/tkstubs/tkStubLib.c",
     "tests/check-package-version.c",
     "tests/unit_tests/lib/common/command_line.c",
     "windows/gvedit/Application.h",
