@@ -8,46 +8,19 @@ import os
 import platform
 import subprocess
 import sys
-from pathlib import Path
-from typing import Dict
 
 import pytest
 
 sys.path.append(os.path.join(os.path.dirname(__file__), "../tests"))
 from gvtest import (  # pylint: disable=wrong-import-position
     dot,
+    freedesktop_os_release,
     is_cmake,
     is_mingw,
     which,
 )
 
 
-def _freedesktop_os_release() -> Dict[str, str]:
-    """
-    polyfill for `platform.freedesktop_os_release`
-    """
-    release = {}
-    os_release = Path("/etc/os-release")
-    if os_release.exists():
-        with open(os_release, "rt", encoding="utf-8") as f:
-            for line in f.readlines():
-                if line.startswith("#") or "=" not in line:
-                    continue
-                key, _, value = (x.strip() for x in line.partition("="))
-                # remove quotes
-                if len(value) >= 2 and value[0] == '"' and value[-1] == '"':
-                    value = value[1:-1]
-                release[key] = value
-    return release
-
-
-def is_centos() -> bool:
-    """
-    is the current CI environment CentOS-based?
-    """
-    return _freedesktop_os_release().get("ID") == "centos"
-
-
 def is_win64() -> bool:
     """
     is the Graphviz under test targeting the x64 Windows API?
@@ -120,12 +93,7 @@ def test_existence(binary: str):
         "vimdot",
     ]
 
-    tools_not_built_with_autotools_on_macos = [
-        "mingle",
-        "smyrna",
-    ]
-
-    os_id = _freedesktop_os_release().get("ID")
+    os_id = freedesktop_os_release().get("ID")
 
     # FIXME: Remove skip when
     # https://gitlab.com/graphviz/graphviz/-/issues/1834 is fixed
@@ -140,14 +108,6 @@ def test_existence(binary: str):
             check_that_tool_does_not_exist(binary, os_id)
             pytest.skip(f"{binary} is not built with MSBuild (#1837)")
 
-    # FIXME: Remove skip when
-    # https://gitlab.com/graphviz/graphviz/-/issues/1854 is fixed
-    if os.getenv("build_system") == "autotools":
-        if platform.system() == "Darwin":
-            if binary in tools_not_built_with_autotools_on_macos:
-                check_that_tool_does_not_exist(binary, os_id)
-                pytest.skip(f"{binary} is not built with autotools on macOS (#1854)")
-
     if binary == "mingle" and is_cmake() and (is_win64() or is_mingw()):
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip(f"{binary} is not built on some Windows due to lacking libANN")
@@ -156,7 +116,7 @@ def test_existence(binary: str):
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip(f"{binary} is not built on 64-bit Windows due to lacking libgd")
 
-    if binary == "gvedit" and platform.system() == "Windows":
+    if binary == "gvedit" and platform.system() == "Windows" and not is_mingw():
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip(f"{binary} is not built on Windows due to lacking Qt")
 
