@@ -25,6 +25,7 @@
 #include <limits.h>
 #include <float.h>
 #include <math.h>
+#include <stdbool.h>
 #include <stddef.h>
 
 static xdot *parseXdotwithattrs(void *e)
@@ -71,7 +72,7 @@ static void set_boundaries(Agraph_t * g)
     view->bdyBottom = bottom;
 }
 
-static void draw_xdot(xdot* x,float base_z)
+static void draw_xdot(xdot* x, double base_z)
 {
 	sdot_op *op;
 	if (!x)
@@ -318,7 +319,7 @@ static void renderNodes(Agraph_t * g)
 
     x=parseXdotwithattrs(g);
     if (x) {
-	draw_xdot(x,-0.2f);
+	draw_xdot(x, -0.2);
 	freeXDot (x);
     }
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) 
@@ -326,7 +327,7 @@ static void renderNodes(Agraph_t * g)
 	    if(!object_color(v,&c))
 		continue;
 	    x=parseXdotwithattrs(v);
-	    draw_xdot(x,-0.1f);
+	    draw_xdot(x, -0.1);
 
 	    if(x)
 		freeXDot (x);
@@ -837,8 +838,8 @@ void updateSmGraph(Agraph_t * g,topview* t)
 	t->Nodecount++;
 
     }
-    aginit(g, AGNODE, "nodeRec", sizeof(nodeRec), 0);
-    aginit(g, AGEDGE, "edgeRec", sizeof(edgeRec), 0);
+    aginit(g, AGNODE, "nodeRec", sizeof(nodeRec), false);
+    aginit(g, AGEDGE, "edgeRec", sizeof(edgeRec), false);
 
     set_boundaries(g);
     view->Topview=t;
@@ -857,7 +858,6 @@ void initSmGraph(Agraph_t * g,topview* rv)
 {
     /*create attribute list*/
     rv->attributes=load_attr_list(view->g[view->activeGraph]);
-    rv->filtered_attr_list=NULL;
 
     // set topological fisheye to NULL
     rv->fisheyeParams.h = NULL;
