@@ -19,8 +19,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/exit.h>
+#include <cgraph/unreachable.h>
 #include <ingraphs/ingraphs.h>
 #include <getopt.h>
 
@@ -85,11 +87,11 @@ static int cmpf(Dt_t * d, void *key1, void *key2, Dtdisc_t * disc)
     (void)d;
     (void)disc;
 
-    double t;
-    t = getdist((Agnode_t *) key1) - getdist((Agnode_t *) key2);
-    if (t < 0)
+    double dist1 = getdist((Agnode_t *) key1);
+    double dist2 = getdist((Agnode_t *) key2);
+    if (dist1 < dist2)
 	return -1;
-    if (t > 0)
+    if (dist1 > dist2)
 	return 1;
     if (key1 < key2)
 	return -1;
@@ -105,9 +107,7 @@ static Dtdisc_t MyDisc = {
     0,				/* Dtmake_f makef */
     0,				/* Dtfree_f freef */
     cmpf,			/* Dtcompar_f comparf */
-    0,				/* Dthash_f hashf */
     0,				/* Dtmemory_f memoryf */
-    0				/* Dtevent_f eventf */
 };
 
 static Agnode_t *extract_min(Dict_t * Q)
@@ -138,7 +138,7 @@ static void update(Dict_t * Q, Agnode_t * dest, Agnode_t * src, double len)
 static void pre(Agraph_t * g)
 {
     len_sym = agattr(g, AGEDGE, "len", NULL);
-    aginit(g, AGNODE, "dijkstra", sizeof(nodedata_t), 1);
+    aginit(g, AGNODE, "dijkstra", sizeof(nodedata_t), true);
 }
 
 static void post(Agraph_t * g)
@@ -260,6 +260,8 @@ static void init(int argc, char *argv[])
 		usage(1);
 	    }
 	    break;
+	default:
+	    UNREACHABLE();
 	}
     }
     argv += optind;
@@ -269,8 +271,8 @@ static void init(int argc, char *argv[])
 	fprintf(stderr, "%s: no node specified\n", CmdName);
 	usage(1);
     }
-    Files = calloc(sizeof(char *), (size_t)argc / 2 + 2);
-    Nodes = calloc(sizeof(char *), (size_t)argc / 2 + 2);
+    Files = gv_calloc((size_t)argc / 2 + 2, sizeof(char *));
+    Nodes = gv_calloc((size_t)argc / 2 + 2, sizeof(char *));
     for (j = i = 0; i < argc; i++) {
 	Nodes[j] = argv[i++];
 	Files[j] = argv[i] ? argv[i] : "-";
@@ -279,11 +281,6 @@ static void init(int argc, char *argv[])
     Nodes[j] = Files[j] = NULL;
 }
 
-static Agraph_t *gread(FILE * fp)
-{
-    return agread(fp, NULL);
-}
-
 int main(int argc, char **argv)
 {
     Agraph_t *g;
@@ -294,7 +291,7 @@ int main(int argc, char **argv)
     Dict_t *Q;
 
     init(argc, argv);
-    newIngraph(&ig, Files, gread);
+    newIngraph(&ig, Files);
 
     Q = dtopen(&MyDisc, Dtoset);
     while ((g = nextGraph(&ig)) != 0) {
