@@ -23,11 +23,15 @@
 #define	NC	3		/* size of HSB color vector */
 
 #include <assert.h>
+#include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/exit.h>
+#include "colorxlate.h"
 #include <math.h>
+#include <stdbool.h>
 #include <stdlib.h>
-typedef struct Agnodeinfo_t {
+typedef struct {
     Agrec_t h;
     double relrank;	/* coordinate of its rank, smaller means lower rank */
     double x[NC];	/* color vector */
@@ -49,8 +53,6 @@ int AdjustSaturation;
 double MinRankSaturation;
 double MaxRankSaturation;
 
-extern char *colorxlate(char *str, char *buf);
-
 static int cmpf(Agnode_t ** n0, Agnode_t ** n1)
 {
     double relrank0 = ND_relrank(*n0);
@@ -64,11 +66,12 @@ static int cmpf(Agnode_t ** n0, Agnode_t ** n1)
 
 static void setcolor(char *p, double *v)
 {
-    char buf[64];
+    agxbuf buf = {0};
     if (sscanf(p, "%lf %lf %lf", v, v + 1, v + 2) != 3 && p[0]) {
-	colorxlate(p, buf);
-	sscanf(buf, "%lf %lf %lf", v, v + 1, v + 2);
+	colorxlate(p, &buf);
+	sscanf(agxbuse(&buf), "%lf %lf %lf", v, v + 1, v + 2);
     }
+    agxbfree(&buf);
 }
 
 static char **Files;
@@ -125,7 +128,7 @@ static void color(Agraph_t * g)
 		"graph must be run through 'dot' before 'gvcolor'\n");
 	graphviz_exit(1);
     }
-    aginit(g, AGNODE, "nodeinfo", sizeof(Agnodeinfo_t), TRUE);
+    aginit(g, AGNODE, "nodeinfo", sizeof(Agnodeinfo_t), true);
     if (agattr(g, AGNODE, "style", 0) == NULL)
 	agattr(g, AGNODE, "style", "filled");
     if ((p = agget(g, "Defcolor")))
@@ -147,7 +150,7 @@ static void color(Agraph_t * g)
     nn = agnnodes(g);
     assert(nn >= 0);
     size_t nnodes = (size_t)nn;
-    nlist = malloc(nnodes * sizeof(Agnode_t *));
+    nlist = gv_calloc(nnodes, sizeof(Agnode_t *));
     size_t i = 0;
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	nlist[i++] = n;
@@ -234,18 +237,13 @@ static void color(Agraph_t * g)
     free (nlist);
 }
 
-static Agraph_t *gread(FILE * fp)
-{
-    return agread(fp, NULL);
-}
-
 int main(int argc, char **argv)
 {
     Agraph_t *g;
     ingraph_state ig;
 
     init(argc, argv);
-    newIngraph(&ig, Files, gread);
+    newIngraph(&ig, Files);
 
     while ((g = nextGraph(&ig)) != 0) {
 	color(g);
