@@ -8,6 +8,7 @@
  * Contributors: See CVS logs. Details at http://www.graphviz.org/
  *************************************************************************/
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "cgraph.h"
@@ -94,8 +95,8 @@ static void nodeInduce(Agraph_t * g, Agraph_t * map)
 
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 		for (e = agfstout(rootg, n); e; e = agnxtout(rootg, e)) {
-			if (agsubnode(g, aghead(e), FALSE))
-				agsubedge(g, e, TRUE);
+			if (agsubnode(g, aghead(e), 0))
+				agsubedge(g, e, 1);
 			else {
 				Agraph_t *tscc = getscc(agtail(e));
 				Agraph_t *hscc = getscc(aghead(e));
@@ -141,7 +142,7 @@ static int visit(Agnode_t * n, Agraph_t * map, Stack * sp, sccstate * st)
 			setrep(subg, agnode(map, name, TRUE));
 			do {
 				t = pop(sp);
-				agsubnode(subg, t, TRUE);
+				agsubnode(subg, t, 1);
 				setval(t, INF);
 				setscc(t, subg);
 				st->N_nodes_in_nontriv_SCC++;
@@ -193,7 +194,7 @@ countComponents(Agraph_t * g, int *max_degree, float *nontree_frac)
 	if (max_degree) {
 		int maxd = 0;
 		for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-			deg = agdegree(g, n, TRUE, TRUE);
+			deg = agdegree(g, n, 1, 1);
 			if (maxd < deg)
 				maxd = deg;
 			setval(n, 0);
@@ -219,8 +220,8 @@ static void process(Agraph_t * G)
 	Stack stack;
 	sccstate state;
 
-	aginit(G, AGRAPH, "scc_graph", sizeof(Agraphinfo_t), TRUE);
-	aginit(G, AGNODE, "scc_node", sizeof(Agnodeinfo_t), TRUE);
+	aginit(G, AGRAPH, "scc_graph", sizeof(Agraphinfo_t), true);
+	aginit(G, AGNODE, "scc_node", sizeof(Agnodeinfo_t), true);
 	state.Comp = state.ID = 0;
 	state.N_nodes_in_nontriv_SCC = 0;
 
@@ -319,18 +320,13 @@ static void scanArgs(int argc, char **argv)
 		outfp = stdout;			/* stdout the default */
 }
 
-static Agraph_t *gread(FILE * fp)
-{
-	return agread(fp, NULL);
-}
-
 int main(int argc, char **argv)
 {
 	Agraph_t *g;
 	ingraph_state ig;
 
 	scanArgs(argc, argv);
-	newIngraph(&ig, Files, gread);
+	newIngraph(&ig, Files);
 
 	while ((g = nextGraph(&ig)) != 0) {
 		if (agisdirected(g))
