@@ -1,3 +1,7 @@
+/// @file
+/// @brief demonstrates usage of @ref aginit, @ref ccomps, @ref nodeInduce,
+/// @ref pack_graph and @ref agdelete
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
@@ -10,8 +14,10 @@
 
 #include <graphviz/gvc.h>
 #include <graphviz/pack.h>
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 int main(int argc, char *argv[]) {
   GVC_t *gvc = gvContext();
@@ -23,6 +29,9 @@ int main(int argc, char *argv[]) {
     fp = stdin;
   graph_t *g = agread(fp, NULL);
 
+  aginit(g, AGRAPH, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
+  aginit(g, AGNODE, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
+
   int ncc;
   graph_t **cc = ccomps(g, &ncc, NULL);
 
@@ -40,6 +49,7 @@ int main(int argc, char *argv[]) {
     gvFreeLayout(gvc, sg);
     agdelete(g, sg);
   }
+  free(cc);
 
   agclose(g);
 
