@@ -11,6 +11,7 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <cgraph/alloc.h>
 #include <cgraph/cghdr.h>
 
 #define MAX(a,b)	((a)>(b)?(a):(b))
@@ -37,14 +38,14 @@ agerrlevel_t agseterr(agerrlevel_t lvl)
     return oldv;
 }
 
-char *aglasterr()
+char *aglasterr(void)
 {
     if (!agerrout)
 	return 0;
     fflush(agerrout);
     long endpos = ftell(agerrout);
     size_t len = (size_t)(endpos - aglast);
-    char *buf = malloc(len + 1);
+    char *buf = gv_alloc(len + 1);
     fseek(agerrout, aglast, SEEK_SET);
     len = fread(buf, sizeof(char), len, agerrout);
     buf[len] = '\0';
@@ -172,9 +173,9 @@ void agwarningf(const char *fmt, ...)
     va_end(args);
 }
 
-int agerrors() { return agmaxerr; }
+int agerrors(void) { return agmaxerr; }
 
-int agreseterrors() 
+int agreseterrors(void)
 { 
     int rc = agmaxerr;
     agmaxerr = 0;
