@@ -8,10 +8,10 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <assert.h>
 #include <cgraph/cghdr.h>
 #include <stddef.h>
-
-static Agtag_t Tag;		/* to silence warnings about initialization */
+#include <stdbool.h>
 
 /* return first outedge of <n> */
 Agedge_t *agfstout(Agraph_t * g, Agnode_t * n)
@@ -128,9 +128,8 @@ static Agedge_t *agfindedge_by_key(Agraph_t * g, Agnode_t * t, Agnode_t * h,
 static Agedge_t *agfindedge_by_id(Agraph_t * g, Agnode_t * t, Agnode_t * h,
                   IDTYPE id)
 {
-    Agtag_t tag;
+    Agtag_t tag = {0};
 
-    tag = Tag;
     tag.objtype = AGEDGE;
     tag.id = id;
     return agfindedge_by_key(g, t, h, tag);
@@ -158,7 +157,7 @@ static void ins(Dict_t * d, Dtlink_t ** set, Agedge_t * e)
 static void del(Dict_t * d, Dtlink_t ** set, Agedge_t * e)
 {
     void *x;
-    NOTUSED(x);
+    (void)x;
     dtrestore(d, *set);
     x = dtdelete(d, e);
     assert(x);
@@ -198,24 +197,24 @@ static Agedge_t *newedge(Agraph_t * g, Agnode_t * t, Agnode_t * h,
 {
     Agedgepair_t *e2;
     Agedge_t *in, *out;
-    int seq;
 
-    (void)agsubnode(g,t,TRUE);
-    (void)agsubnode(g,h,TRUE);
+    (void)agsubnode(g, t, 1);
+    (void)agsubnode(g, h, 1);
     e2 = agalloc(g, sizeof(Agedgepair_t));
     in = &(e2->in);
     out = &(e2->out);
-    seq = agnextseq(g, AGEDGE);
+    uint64_t seq = agnextseq(g, AGEDGE);
+    assert((seq & SEQ_MASK) == seq && "sequence ID overflow");
     AGTYPE(in) = AGINEDGE;
     AGTYPE(out) = AGOUTEDGE;
     AGID(in) = AGID(out) = id;
-    AGSEQ(in) = AGSEQ(out) = seq;
+    AGSEQ(in) = AGSEQ(out) = seq & SEQ_MASK;
     in->node = t;
     out->node = h;
 
     installedge(g, out);
     if (g->desc.has_attrs) {
-	(void) agbindrec(out, AgDataRecName, sizeof(Agattr_t), FALSE);
+	(void)agbindrec(out, AgDataRecName, sizeof(Agattr_t), false);
 	agedgeattr_init(g, out);
     }
     agmethod_init(g, out);
@@ -225,11 +224,10 @@ static Agedge_t *newedge(Agraph_t * g, Agnode_t * t, Agnode_t * h,
 /* edge creation predicate */
 static int ok_to_make_edge(Agraph_t * g, Agnode_t * t, Agnode_t * h)
 {
-    Agtag_t key;
+    Agtag_t key = {0};
 
     /* protect against self, multi-edges in strict graphs */
     if (agisstrict(g)) {
-	key = Tag;
 	key.objtype = 0;	/* wild card */
 	if (agfindedge_by_key(g, t, h, key))
 	    return FALSE;
@@ -266,13 +264,11 @@ Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h, char *name,
 {
     Agedge_t *e;
     IDTYPE my_id;
-    int have_id;
 
-    have_id = agmapnametoid(g, AGEDGE, name, &my_id, FALSE);
-    if (have_id || (name == NULL && (NOT(cflag) || agisstrict(g)))) {
+    int have_id = agmapnametoid(g, AGEDGE, name, &my_id, false);
+    if (have_id || (name == NULL && (!cflag || agisstrict(g)))) {
 	/* probe for pre-existing edge */
-	Agtag_t key;
-	key = Tag;
+	Agtag_t key = {0};
 	if (have_id) {
 	    key.id = my_id;
 	    key.objtype = AGEDGE;
@@ -298,7 +294,7 @@ Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h, char *name,
     }
 
     if (cflag && ok_to_make_edge(g, t, h)
-	&& agmapnametoid(g, AGEDGE, name, &my_id, TRUE)) { /* reserve id */
+	&& agmapnametoid(g, AGEDGE, name, &my_id, true)) { /* reserve id */
 	e = newedge(g, t, h, my_id);
 	agregister(g, AGEDGE, e); /* register new object in external namespace */
     }
@@ -313,7 +309,7 @@ void agdeledgeimage(Agraph_t * g, Agedge_t * e, void *ignored)
     Agnode_t *t, *h;
     Agsubnode_t *sn;
 
-    NOTUSED(ignored);
+    (void)ignored;
     if (AGTYPE(e) == AGINEDGE) {
 	in = e;
 	out = AGIN2OUT(e);
@@ -383,10 +379,10 @@ static int agedgeidcmpf(Dict_t * d, void *arg_e0, void *arg_e1, Dtdisc_t * disc)
 {
     Agedge_t *e0, *e1;
 
-    NOTUSED(d);
+    (void)d;
     e0 = arg_e0;
     e1 = arg_e1;
-    NOTUSED(disc);
+    (void)disc;
 
     if (AGID(e0->node) < AGID(e1->node)) return -1;
     if (AGID(e0->node) > AGID(e1->node)) return 1;
@@ -403,10 +399,10 @@ static int agedgeseqcmpf(Dict_t * d, void *arg_e0, void *arg_e1, Dtdisc_t * disc
 {
     Agedge_t *e0, *e1;
 
-    NOTUSED(d);
+    (void)d;
     e0 = arg_e0;
     e1 = arg_e1;
-    NOTUSED(disc);
+    (void)disc;
     assert(arg_e0 && arg_e1);
 
     if (e0->node != e1->node) {
@@ -422,52 +418,28 @@ static int agedgeseqcmpf(Dict_t * d, void *arg_e0, void *arg_e1, Dtdisc_t * disc
 
 /* indexing for ordered traversal */
 Dtdisc_t Ag_mainedge_seq_disc = {
-    0,				/* pass object ptr      */
-    0,				/* size (ignored)       */
-    offsetof(Agedge_t,seq_link),/* use internal links	*/
-    NIL(Dtmake_f),
-    NIL(Dtfree_f),
-    agedgeseqcmpf,
-    NIL(Dthash_f),
-    agdictobjmem,
-    NIL(Dtevent_f)
+    .link = offsetof(Agedge_t, seq_link), // use internal links
+    .comparf = agedgeseqcmpf,
+    .memoryf = agdictobjmem,
 };
 
 Dtdisc_t Ag_subedge_seq_disc = {
-    0,				/* pass object ptr      */
-    0,				/* size (ignored)       */
-    -1,				/* use external holder objects */
-    NIL(Dtmake_f),
-    NIL(Dtfree_f),
-    agedgeseqcmpf,
-    NIL(Dthash_f),
-    agdictobjmem,
-    NIL(Dtevent_f)
+    .link = -1, // use external holder objects
+    .comparf = agedgeseqcmpf,
+    .memoryf = agdictobjmem,
 };
 
 /* indexing for random search */
 Dtdisc_t Ag_mainedge_id_disc = {
-    0,				/* pass object ptr      */
-    0,				/* size (ignored)       */
-    offsetof(Agedge_t,id_link),	/* use internal links	*/
-    NIL(Dtmake_f),
-    NIL(Dtfree_f),
-    agedgeidcmpf,
-    NIL(Dthash_f),
-    agdictobjmem,
-    NIL(Dtevent_f)
+    .link = offsetof(Agedge_t, id_link), // use internal links
+    .comparf = agedgeidcmpf,
+    .memoryf = agdictobjmem,
 };
 
 Dtdisc_t Ag_subedge_id_disc = {
-    0,				/* pass object ptr      */
-    0,				/* size (ignored)       */
-    -1,				/* use external holder objects */
-    NIL(Dtmake_f),
-    NIL(Dtfree_f),
-    agedgeidcmpf,
-    NIL(Dthash_f),
-    agdictobjmem,
-    NIL(Dtevent_f)
+    .link = -1, // use external holder objects
+    .comparf = agedgeidcmpf,
+    .memoryf = agdictobjmem,
 };
 
 /* expose macros as functions for ease of debugging
