@@ -2,8 +2,14 @@
 
 #include <assert.h>
 #include <cgraph/alloc.h>
+#include <cgraph/exit.h>
+#include <cgraph/likely.h>
+#include <errno.h>
 #include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #ifdef __GNUC__
 #define LIST_UNUSED __attribute__((unused))
@@ -37,18 +43,6 @@
     size_t capacity; /* available storage slots */                             \
   } name##_t;                                                                  \
                                                                                \
-  /** create a new list                                                        \
-   *                                                                           \
-   * This function is provided for convenience, but it the equivalent of zero  \
-   * initialization, `list_t list = {0}`. In general, the latter should be     \
-   * preferred as it can be understood locally.                                \
-   *                                                                           \
-   * \return A new empty list                                                  \
-   */                                                                          \
-  static inline LIST_UNUSED name##_t name##_new(void) {                        \
-    return (name##_t){0};                                                      \
-  }                                                                            \
-                                                                               \
   /** get the number of elements in a list */                                  \
   static inline LIST_UNUSED size_t name##_size(const name##_t *list) {         \
     assert(list != NULL);                                                      \
@@ -61,18 +55,43 @@
     return name##_size(list) == 0;                                             \
   }                                                                            \
                                                                                \
-  static inline LIST_UNUSED void name##_append(name##_t *list, type item) {    \
+  static inline int name##_try_append(name##_t *list, type item) {             \
     assert(list != NULL);                                                      \
                                                                                \
     /* do we need to expand the backing storage? */                            \
     if (list->size == list->capacity) {                                        \
-      size_t c = list->capacity == 0 ? 1 : (list->capacity * 2);               \
-      list->data = gv_recalloc(list->data, list->capacity, c, sizeof(type));   \
+      const size_t c = list->capacity == 0 ? 1 : (list->capacity * 2);         \
+                                                                               \
+      /* will the calculation of the new size overflow? */                     \
+      if (UNLIKELY(SIZE_MAX / c < sizeof(type))) {                             \
+        return ERANGE;                                                         \
+      }                                                                        \
+                                                                               \
+      type *data = (type *)realloc(list->data, c * sizeof(type));              \
+      if (UNLIKELY(data == NULL)) {                                            \
+        return ENOMEM;                                                         \
+      }                                                                        \
+                                                                               \
+      /* zero the new memory */                                                \
+      memset((char *)data + list->capacity * sizeof(type), 0,                  \
+             (c - list->capacity) * sizeof(type));                             \
+                                                                               \
+      list->data = data;                                                       \
       list->capacity = c;                                                      \
     }                                                                          \
                                                                                \
     list->data[list->size] = item;                                             \
     ++list->size;                                                              \
+                                                                               \
+    return 0;                                                                  \
+  }                                                                            \
+                                                                               \
+  static inline LIST_UNUSED void name##_append(name##_t *list, type item) {    \
+    int rc = name##_try_append(list, item);                                    \
+    if (rc != 0) {                                                             \
+      fprintf(stderr, "realloc failed: %s\n", strerror(rc));                   \
+      graphviz_exit(EXIT_FAILURE);                                             \
+    }                                                                          \
   }                                                                            \
                                                                                \
   /** retrieve an element from a list                                          \
@@ -105,6 +124,34 @@
     list->data[index] = item;                                                  \
   }                                                                            \
                                                                                \
+  /** remove an element from a list                                            \
+   *                                                                           \
+   * \param list List to operate on                                            \
+   * \param item Value of element to remove                                    \
+   */                                                                          \
+  static inline LIST_UNUSED void name##_remove(name##_t *list,                 \
+                                               const type item) {              \
+    assert(list != NULL);                                                      \
+                                                                               \
+    for (size_t i = 0; i < list->size; ++i) {                                  \
+      /* is this the element we are looking for? */                            \
+      if (memcmp(&list->data[i], &item, sizeof(type)) == 0) {                  \
+                                                                               \
+        /* destroy the element we are about to remove */                       \
+        void (*dtor_)(type) = (void (*)(type))(dtor);                          \
+        if (dtor_ != NULL) {                                                   \
+          dtor_(list->data[i]);                                                \
+        }                                                                      \
+                                                                               \
+        /* shrink the list */                                                  \
+        size_t remainder = (list->size - i - 1) * sizeof(type);                \
+        memmove(&list->data[i], &list->data[i + 1], remainder);                \
+        --list->size;                                                          \
+        return;                                                                \
+      }                                                                        \
+    }                                                                          \
+  }                                                                            \
+                                                                               \
   /** access an element in a list for the purpose of modification              \
    *                                                                           \
    * Because this acquires an internal pointer into the list structure, `get`  \
@@ -137,6 +184,25 @@
     list->size = 0;                                                            \
   }                                                                            \
                                                                                \
+  /** ensure the list can fit a given number of items without reallocation     \
+   *                                                                           \
+   * \param list List to operate on                                            \
+   * \param capacity Number of items the list should be able to contain        \
+   */                                                                          \
+  static inline LIST_UNUSED void name##_reserve(name##_t *list,                \
+                                                size_t capacity) {             \
+    assert(list != NULL);                                                      \
+                                                                               \
+    /* if we can already fit enough items, nothing to do */                    \
+    if (list->capacity >= capacity) {                                          \
+      return;                                                                  \
+    }                                                                          \
+                                                                               \
+    list->data = (type *)gv_recalloc(list->data, list->capacity, capacity,     \
+                                     sizeof(type));                            \
+    list->capacity = capacity;                                                 \
+  }                                                                            \
+                                                                               \
   /** shrink or grow the list to the given size                                \
    *                                                                           \
    * \param list List to operate on                                            \
@@ -164,13 +230,26 @@
     }                                                                          \
   }                                                                            \
                                                                                \
+  /** sort the list using the given comparator */                              \
+  static inline LIST_UNUSED void name##_sort(                                  \
+      name##_t *list, int (*cmp)(const type *a, const type *b)) {              \
+    assert(list != NULL);                                                      \
+    assert(cmp != NULL);                                                       \
+                                                                               \
+    int (*compar)(const void *, const void *) =                                \
+        (int (*)(const void *, const void *))cmp;                              \
+    if (list->size > 0) {                                                      \
+      qsort(list->data, list->size, sizeof(type), compar);                     \
+    }                                                                          \
+  }                                                                            \
+                                                                               \
   /** deallocate unused backing storage, shrinking capacity to size */         \
   static inline LIST_UNUSED void name##_shrink_to_fit(name##_t *list) {        \
     assert(list != NULL);                                                      \
                                                                                \
     if (list->capacity > list->size) {                                         \
-      list->data =                                                             \
-          gv_recalloc(list->data, list->capacity, list->size, sizeof(type));   \
+      list->data = (type *)gv_recalloc(list->data, list->capacity, list->size, \
+                                       sizeof(type));                          \
       list->capacity = list->size;                                             \
     }                                                                          \
   }                                                                            \
@@ -180,7 +259,7 @@
     assert(list != NULL);                                                      \
     name##_clear(list);                                                        \
     free(list->data);                                                          \
-    *list = (name##_t){0};                                                     \
+    memset(list, 0, sizeof(*list));                                            \
   }                                                                            \
                                                                                \
   /** alias for append */                                                      \
@@ -215,7 +294,8 @@
    */                                                                          \
   static inline LIST_UNUSED name##_t name##_attach(type *data, size_t size) {  \
     assert(data != NULL || size == 0);                                         \
-    return (name##_t){.data = data, .size = size, .capacity = size};           \
+    name##_t list = {data, size, size};                                        \
+    return list;                                                               \
   }                                                                            \
                                                                                \
   /** transform a managed list into a bare array                               \
@@ -230,6 +310,6 @@
   static inline LIST_UNUSED type *name##_detach(name##_t *list) {              \
     assert(list != NULL);                                                      \
     type *data = list->data;                                                   \
-    *list = (name##_t){0};                                                     \
+    memset(list, 0, sizeof(*list));                                            \
     return data;                                                               \
   }
