@@ -29,7 +29,7 @@ static Agnode_t *agfindnode_by_name(Agraph_t * g, char *name)
 {
     IDTYPE id;
 
-    if (agmapnametoid(g, AGNODE, name, &id, FALSE))
+    if (agmapnametoid(g, AGNODE, name, &id, false))
 	return agfindnode_by_id(g, id);
     else
 	return NULL;
@@ -128,7 +128,7 @@ Agnode_t *agidnode(Agraph_t * g, IDTYPE id, int cflag)
     if (n == NULL && cflag) {
 	root = agroot(g);
 	if ((g != root) && ((n = agfindnode_by_id(root, id))))	/*old */
-	    agsubnode(g, n, TRUE);	/* insert locally */
+	    agsubnode(g, n, 1);	/* insert locally */
 	else {
 	    if (agallocid(g, AGNODE, id)) {	/* new */
 		n = newnode(g, id, agnextseq(g, AGNODE));
@@ -150,17 +150,17 @@ Agnode_t *agnode(Agraph_t * g, char *name, int cflag)
 
     root = agroot(g);
     /* probe for existing node */
-    if (agmapnametoid(g, AGNODE, name, &id, FALSE)) {
+    if (agmapnametoid(g, AGNODE, name, &id, false)) {
 	if ((n = agfindnode_by_id(g, id)))
 	    return n;
 
 	/* might already exist globally, but need to insert locally */
 	if (cflag && (g != root) && ((n = agfindnode_by_id(root, id)))) {
-	    return agsubnode(g, n, TRUE);
+	    return agsubnode(g, n, 1);
 	}
     }
 
-    if (cflag && agmapnametoid(g, AGNODE, name, &id, TRUE)) {	/* reserve id */
+    if (cflag && agmapnametoid(g, AGNODE, name, &id, true)) {	/* reserve id */
 	n = newnode(g, id, agnextseq(g, AGNODE));
 	installnodetoroot(g, n);
 	initnode(g, n);
@@ -240,7 +240,7 @@ int agrelabel_node(Agnode_t * n, char *newname)
     g = agroot(agraphof(n));
     if (agfindnode_by_name(g, newname))
 	return FAILURE;
-    if (agmapnametoid(g, AGNODE, newname, &new_id, TRUE)) {
+    if (agmapnametoid(g, AGNODE, newname, &new_id, true)) {
 	if (agfindnode_by_id(agroot(g), new_id) == NULL) {
 	    agfreeid(g, AGNODE, AGID(n));
 	    agapply(g, (Agobj_t*)n, (agobjfn_t)dict_relabel, &new_id, FALSE);
@@ -308,10 +308,7 @@ static int agsubnodeseqcmpf(Dict_t * d, void *arg0, void *arg1, Dtdisc_t * disc)
  * dictionaries use the same subnode object, so only one
  * should do the deletion.
  */
-static void
-free_subnode (Dt_t* d, Agsubnode_t* sn, Dtdisc_t * disc)
-{
-   (void)d; /* unused */
+static void free_subnode(Agsubnode_t *sn, Dtdisc_t *disc) {
    (void)disc; /* unused */
    if (!AGSNMAIN(sn)) 
 	agfree (sn->node->root, sn);
