@@ -10,7 +10,8 @@
 
 #include <stdio.h>
 
-
+#include <math.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -20,7 +21,10 @@
 #include <common/colorprocs.h>
 #include <common/colortbl.h>
 #include <common/memory.h>
+#include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
 #include <cgraph/strcasecmp.h>
+#include <cgraph/unreachable.h>
 
 static char* colorscheme;
 
@@ -41,8 +45,8 @@ static void hsv2rgb(double h, double s, double v,
 	i = (int) h;
 	f = h - (double) i;
 	p = v * (1 - s);
-	q = v * (1 - (s * f));
-	t = v * (1 - (s * (1 - f)));
+	q = v * (1 - s * f);
+	t = v * (1 - s * (1 - f));
 	switch (i) {
 	case 0:
 	    *r = v;
@@ -74,6 +78,8 @@ static void hsv2rgb(double h, double s, double v,
 	    *g = p;
 	    *b = q;
 	    break;
+	default:
+	    UNREACHABLE();
 	}
     }
 }
@@ -86,8 +92,8 @@ static void rgb2hsv(double r, double g, double b,
     double rc, bc, gc;
     double ht = 0.0, st = 0.0;
 
-    rgbmin = MIN(r, MIN(g, b));
-    rgbmax = MAX(r, MAX(g, b));
+    rgbmin = fmin(r, fmin(g, b));
+    rgbmax = fmax(r, fmax(g, b));
 
     if (rgbmax > 0.0)
 	st = (rgbmax - rgbmin) / rgbmax;
@@ -117,8 +123,8 @@ static void rgb2cmyk(double r, double g, double b, double *c, double *m,
     *c = 1.0 - r;
     *m = 1.0 - g;
     *y = 1.0 - b;
-    *k = *c < *m ? *c : *m;
-    *k = *y < *k ? *y : *k;
+    *k = fmin(*c, *m);
+    *k = fmin(*y, *k);
     *c -= *k;
     *m -= *k;
     *y -= *k;
@@ -131,46 +137,34 @@ static int colorcmpf(const void *p0, const void *p1)
 
 char *canontoken(char *str)
 {
-    static unsigned char *canon;
+    static char *canon;
     static size_t allocated;
-    unsigned char c, *p, *q;
+    char c, *p, *q;
     size_t len;
 
-    p = (unsigned char *) str;
+    p = str;
     len = strlen(str);
     if (len >= allocated) {
-	allocated = len + 1 + 10;
-	canon = grealloc(canon, allocated);
-	if (!canon)
-	    return NULL;
+	size_t new_allocated = len + 1 + 10;
+	canon = gv_recalloc(canon, allocated, new_allocated, sizeof(char));
+	allocated = new_allocated;
     }
-    q = (unsigned char *) canon;
+    q = canon;
     while ((c = *p++)) {
-	/* if (isalnum(c) == FALSE) */
-	    /* continue; */
 	if (isupper(c))
-	    c = (unsigned char) tolower(c);
+	    c = (char)tolower(c);
 	*q++ = c;
     }
     *q = '\0';
-    return (char*)canon;
+    return canon;
 }
 
 /* fullColor:
  * Return "/prefix/str"
  */
-static char* fullColor (char* prefix, char* str)
-{
-    static char *fulls;
-    static size_t allocated;
-    size_t len = strlen(prefix) + strlen(str) + 3;
-
-    if (len >= allocated) {
-	allocated = len + 10;
-	fulls = grealloc(fulls, allocated);
-    }
-    sprintf (fulls, "/%s/%s", prefix, str);
-    return fulls;
+static char *fullColor(agxbuf *xb, const char *prefix, const char *str) {
+  agxbprint(xb, "/%s/%s", prefix, str);
+  return agxbuse(xb);
 }
 
 /* resolveColor:
@@ -218,13 +212,14 @@ static char* resolveColor (char* str)
     if (!strcmp(str, "black")) return str;
     if (!strcmp(str, "white")) return str;
     if (!strcmp(str, "lightgrey")) return str;
+    agxbuf xb = {0};
     if (*str == '/') {   /* if begins with '/' */
 	c2 = str+1;
         if ((ss = strchr(c2, '/'))) {  /* if has second '/' */
 	    if (*c2 == '/') {    /* if second '/' is second character */
 		    /* Do not compare against final '/' */
 		if (ISNONDFLT(colorscheme))
-		    s = fullColor (colorscheme, c2+1);
+		    s = fullColor(&xb, colorscheme, c2+1);
 		else
 		    s = c2+1;
 	    }
@@ -233,36 +228,43 @@ static char* resolveColor (char* str)
 	}
 	else s = c2;
     }
-    else if (ISNONDFLT(colorscheme)) s = fullColor (colorscheme, str);
+    else if (ISNONDFLT(colorscheme)) s = fullColor(&xb, colorscheme, str);
     else s = str;
-    return canontoken(s);
+    s = canontoken(s);
+    agxbfree(&xb);
+    return s;
 }
 
 int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 {
     static hsvrgbacolor_t *last;
-    static unsigned char *canon;
-    static size_t allocated;
-    unsigned char *p, *q;
+    char *p;
     hsvrgbacolor_t fake;
-    unsigned char c;
+    char c;
     double H, S, V, A, R, G, B;
     double C, M, Y, K;
     unsigned int r, g, b, a;
-    size_t len;
     int rc;
 
     color->type = target_type;
 
     rc = COLOR_OK;
     for (; *str == ' '; str++);	/* skip over any leading whitespace */
-    p = (unsigned char *) str;
+    p = str;
 
     /* test for rgb value such as: "#ff0000"
        or rgba value such as "#ff000080" */
     a = 255;			/* default alpha channel value=opaque in case not supplied */
-    if ((*p == '#')
-	&& (sscanf((char *) p, "#%2x%2x%2x%2x", &r, &g, &b, &a) >= 3)) {
+    bool is_rgb = *p == '#' && sscanf(p, "#%2x%2x%2x%2x", &r, &g, &b, &a) >= 3;
+    if (!is_rgb) { // try 3 letter form
+	is_rgb = *p == '#' && strlen(p) == 4 && sscanf(p, "#%1x%1x%1x", &r, &g, &b) == 3;
+	if (is_rgb) {
+	    r |= r << 4;
+	    g |= g << 4;
+	    b |= b << 4;
+	}
+    }
+    if (is_rgb) {
 	switch (target_type) {
 	case HSVA_DOUBLE:
 	    R = (double) r / 255.0;
@@ -276,10 +278,10 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    color->u.HSVA[3] = A;
 	    break;
 	case RGBA_BYTE:
-	    color->u.rgba[0] = r;
-	    color->u.rgba[1] = g;
-	    color->u.rgba[2] = b;
-	    color->u.rgba[3] = a;
+	    color->u.rgba[0] = (unsigned char)r;
+	    color->u.rgba[1] = (unsigned char)g;
+	    color->u.rgba[2] = (unsigned char)b;
+	    color->u.rgba[3] = (unsigned char)a;
 	    break;
 	case CMYK_BYTE:
 	    R = (double) r / 255.0;
@@ -292,10 +294,10 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    color->u.cmyk[3] = (int) K *255;
 	    break;
 	case RGBA_WORD:
-	    color->u.rrggbbaa[0] = r * 65535 / 255;
-	    color->u.rrggbbaa[1] = g * 65535 / 255;
-	    color->u.rrggbbaa[2] = b * 65535 / 255;
-	    color->u.rrggbbaa[3] = a * 65535 / 255;
+	    color->u.rrggbbaa[0] = (int)(r * 65535 / 255);
+	    color->u.rrggbbaa[1] = (int)(g * 65535 / 255);
+	    color->u.rrggbbaa[2] = (int)(b * 65535 / 255);
+	    color->u.rrggbbaa[3] = (int)(a * 65535 / 255);
 	    break;
 	case RGBA_DOUBLE:
 	    color->u.RGBA[0] = (double) r / 255.0;
@@ -307,47 +309,39 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    break;
 	case COLOR_INDEX:
 	    break;
+	default:
+	    UNREACHABLE();
 	}
 	return rc;
     }
 
     /* test for hsv value such as: ".6,.5,.3" */
-    if (((c = *p) == '.') || isdigit(c)) {
-	len = strlen((char*)p);
-	if (len >= allocated) {
-	    allocated = len + 1 + 10;
-	    canon = grealloc(canon, allocated);
-	    if (! canon) {
-		rc = COLOR_MALLOC_FAIL;
-		return rc;
-	    }
-	}
-	q = canon;
+    if ((c = *p) == '.' || isdigit(c)) {
+	agxbuf canon = {0};
 	while ((c = *p++)) {
-	    if (c == ',')
-		c = ' ';
-	    *q++ = c;
+	    agxbputc(&canon, c == ',' ? ' ' : c);
 	}
-	*q = '\0';
 
-	if (sscanf((char *) canon, "%lf%lf%lf", &H, &S, &V) == 3) {
+	A = 1.0; // default
+	if (sscanf(agxbuse(&canon), "%lf%lf%lf%lf", &H, &S, &V, &A) >= 3) {
 	    /* clip to reasonable values */
-	    H = MAX(MIN(H, 1.0), 0.0);
-	    S = MAX(MIN(S, 1.0), 0.0);
-	    V = MAX(MIN(V, 1.0), 0.0);
+	    H = fmax(fmin(H, 1.0), 0.0);
+	    S = fmax(fmin(S, 1.0), 0.0);
+	    V = fmax(fmin(V, 1.0), 0.0);
+	    A = fmax(fmin(A, 1.0), 0.0);
 	    switch (target_type) {
 	    case HSVA_DOUBLE:
 		color->u.HSVA[0] = H;
 		color->u.HSVA[1] = S;
 		color->u.HSVA[2] = V;
-		color->u.HSVA[3] = 1.0; /* opaque */
+		color->u.HSVA[3] = A;
 		break;
 	    case RGBA_BYTE:
 		hsv2rgb(H, S, V, &R, &G, &B);
 		color->u.rgba[0] = (int) (R * 255);
 		color->u.rgba[1] = (int) (G * 255);
 		color->u.rgba[2] = (int) (B * 255);
-		color->u.rgba[3] = 255;	/* opaque */
+		color->u.rgba[3] = (int) (A * 255);
 		break;
 	    case CMYK_BYTE:
 		hsv2rgb(H, S, V, &R, &G, &B);
@@ -362,36 +356,35 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 		color->u.rrggbbaa[0] = (int) (R * 65535);
 		color->u.rrggbbaa[1] = (int) (G * 65535);
 		color->u.rrggbbaa[2] = (int) (B * 65535);
-		color->u.rrggbbaa[3] = 65535;	/* opaque */
+		color->u.rrggbbaa[3] = (int) (A * 65535);
 		break;
 	    case RGBA_DOUBLE:
 		hsv2rgb(H, S, V, &R, &G, &B);
 		color->u.RGBA[0] = R;
 		color->u.RGBA[1] = G;
 		color->u.RGBA[2] = B;
-		color->u.RGBA[3] = 1.0;	/* opaque */
+		color->u.RGBA[3] = A;
 		break;
 	    case COLOR_STRING:
 		break;
 	    case COLOR_INDEX:
 		break;
+	    default:
+		UNREACHABLE();
 	    }
+	    agxbfree(&canon);
 	    return rc;
 	}
+	agxbfree(&canon);
     }
 
     /* test for known color name (generic, not renderer specific known names) */
     fake.name = resolveColor(str);
     if (!fake.name)
 	return COLOR_MALLOC_FAIL;
-    if ((last == NULL)
-	|| (last->name[0] != fake.name[0])
-	|| (strcmp(last->name, fake.name))) {
-	last = (hsvrgbacolor_t *) bsearch((void *) &fake,
-				      (void *) color_lib,
-				      sizeof(color_lib) /
-				      sizeof(hsvrgbacolor_t), sizeof(fake),
-				      colorcmpf);
+    if (last == NULL || strcmp(last->name, fake.name)) {
+	last = bsearch(&fake, color_lib, sizeof(color_lib) / sizeof(hsvrgbacolor_t),
+	               sizeof(fake), colorcmpf);
     }
     if (last != NULL) {
 	switch (target_type) {
@@ -408,9 +401,9 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    color->u.rgba[3] = last->a;
 	    break;
 	case CMYK_BYTE:
-	    R = (last->r) / 255.0;
-	    G = (last->g) / 255.0;
-	    B = (last->b) / 255.0;
+	    R = last->r / 255.0;
+	    G = last->g / 255.0;
+	    B = last->b / 255.0;
 	    rgb2cmyk(R, G, B, &C, &M, &Y, &K);
 	    color->u.cmyk[0] = (int) C * 255;
 	    color->u.cmyk[1] = (int) M * 255;
@@ -433,6 +426,8 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    break;
 	case COLOR_INDEX:
 	    break;
+	default:
+	    UNREACHABLE();
 	}
 	return rc;
     }
@@ -464,83 +459,9 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	break;
     case COLOR_INDEX:
 	break;
-    }
-    return rc;
-}
-
-static void rgba_wordToByte (int* rrggbbaa, unsigned char* rgba)
-{
-    int i;
-
-    for (i = 0; i < 4; i++) {
-	rgba[i] = rrggbbaa[i] * 255 / 65535;
-    }
-}
-
-static void rgba_dblToByte (double* RGBA, unsigned char* rgba)
-{
-    int i;
-
-    for (i = 0; i < 4; i++) {
-	rgba[i] = (unsigned char)(RGBA[i] * 255);
-    }
-}
-
-/* colorCvt:
- * Color format converter. 
- * Except for the trivial case, it converts the input color to a string
- * representation and then calls colorxlate.
- * ncolor must point to a gvcolor_t struct with type specifying the desired
- * output type.
- */
-int colorCvt(gvcolor_t *ocolor, gvcolor_t *ncolor)
-{
-    int rc;
-    char buf[BUFSIZ];
-    char* s;
-    unsigned char rgba[4];
-
-    if (ocolor->type == ncolor->type) {
-	memcpy (&ncolor->u, &ocolor->u, sizeof(ocolor->u));    
-	return COLOR_OK; 
-    }
-    s = buf;
-    switch (ocolor->type) {
-    case HSVA_DOUBLE :
-	snprintf(buf, sizeof(buf), "%.03f %.03f %.03f %.03f",
-	    ocolor->u.HSVA[0], ocolor->u.HSVA[1], ocolor->u.HSVA[2], ocolor->u.HSVA[3]);
-	break;
-    case RGBA_BYTE :
-	snprintf(buf, sizeof(buf), "#%02x%02x%02x%02x",
-	    ocolor->u.rgba[0], ocolor->u.rgba[1], ocolor->u.rgba[2], ocolor->u.rgba[3]);
-	break;
-    case RGBA_WORD:
-	rgba_wordToByte (ocolor->u.rrggbbaa, rgba);
-	snprintf(buf, sizeof(buf), "#%02x%02x%02x%02x", rgba[0], rgba[1], rgba[2],
-	         rgba[3]);
-	break;
-    case RGBA_DOUBLE:
-	rgba_dblToByte (ocolor->u.RGBA, rgba);
-	snprintf(buf, sizeof(buf), "#%02x%02x%02x%02x", rgba[0], rgba[1], rgba[2],
-	         rgba[3]);
-	break;
-    case COLOR_STRING:
-	s = ocolor->u.string;
-	break;
-    case CMYK_BYTE :
-	/* agerr (AGWARN, "Input color type 'CMYK_BYTE' not supported for conversion\n"); */
-	return COLOR_UNKNOWN;
-	break;
-    case COLOR_INDEX:
-	/* agerr (AGWARN, "Input color type 'COLOR_INDEX' not supported for conversion\n"); */
-	return COLOR_UNKNOWN;
-	break;
     default:
-	/* agerr (AGWARN, "Unknown input color type value '%u'\n", ncolor->type); */
-	return COLOR_UNKNOWN;
-	break;
+	UNREACHABLE();
     }
-    rc = colorxlate (s, ncolor, ncolor->type);
     return rc;
 }
 
@@ -551,6 +472,3 @@ void setColorScheme (char* s)
 {
     colorscheme = s;
 }
-
-
-
