@@ -8,8 +8,7 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef GV_MEMORY_H
-#define GV_MEMORY_H
+#pragma once
 
 #include <stdlib.h>
 
@@ -17,35 +16,27 @@
 extern "C" {
 #endif
 
-#define NEW(t)           (t*)zmalloc(sizeof(t))
 #define N_NEW(n,t)       (t*)gcalloc((n),sizeof(t))
-#define GNEW(t)          (t*)gmalloc(sizeof(t))
 
 #define N_GNEW(n,t)      (t*)gcalloc((n),sizeof(t))
-#define N_GGNEW(n,t)      (t*)calloc((n),sizeof(t))
 #define ALLOC(size,ptr,type) (ptr? (type*)grealloc(ptr,(size)*sizeof(type)):(type*)gmalloc((size)*sizeof(type)))
-#define RALLOC(size,ptr,type) ((type*)grealloc(ptr,(size)*sizeof(type)))
-#define ZALLOC(size,ptr,type,osize) (ptr? (type*)zrealloc(ptr,size,sizeof(type),osize):(type*)zmalloc((size)*sizeof(type)))
 #ifdef GVDLL
-#define extern __declspec(dllexport)
+#ifdef GVC_EXPORTS
+#define MEMORY_API __declspec(dllexport)
 #else
-#ifdef _WIN32
-#ifndef GVC_EXPORTS
-#define extern __declspec(dllimport)
+#define MEMORY_API __declspec(dllimport)
 #endif
 #endif
 
+#ifndef MEMORY_API
+#define MEMORY_API /* nothing */
 #endif
 
-    extern void *zmalloc(size_t);
-    extern void *zrealloc(void *, size_t, size_t, size_t);
-    extern void *gcalloc(size_t nmemb, size_t size);
-    extern void *gmalloc(size_t);
-	extern void *grealloc(void *, size_t);
-#undef extern
+    MEMORY_API void *gcalloc(size_t nmemb, size_t size);
+    MEMORY_API void *gmalloc(size_t);
+	MEMORY_API void *grealloc(void *, size_t);
+#undef MEMORY_API
 
 #ifdef __cplusplus
 }
 #endif
-
-#endif
