@@ -9,18 +9,17 @@
  *************************************************************************/
 
 #include <sys/stat.h>
-
+#include <cgraph/alloc.h>
 #include <common/render.h>
 #include <gvc/gvio.h>
 #include <cgraph/strcasecmp.h>
 #include <stdbool.h>
+#include <stdio.h>
 
 static int N_EPSF_files;
 static Dict_t *EPSF_contents;
 
-static void ps_image_free(Dict_t * dict, usershape_t * p, Dtdisc_t * disc)
-{
-    (void)dict;
+static void ps_image_free(usershape_t *p, Dtdisc_t *disc) {
     (void)disc;
     free(p->data);
 }
@@ -33,18 +32,15 @@ static Dtdisc_t ImageDictDisc = {
 
 static usershape_t *user_init(const char *str)
 {
-    char *contents;
     char line[BUFSIZ];
     FILE *fp;
     struct stat statbuf;
-    bool must_inline;
     int lx, ly, ux, uy;
-    usershape_t *us;
 
     if (!EPSF_contents)
 	EPSF_contents = dtopen(&ImageDictDisc, Dtoset);
 
-    us = dtmatch(EPSF_contents, str);
+    usershape_t *us = dtmatch(EPSF_contents, str);
     if (us)
 	return us;
 
@@ -54,18 +50,18 @@ static usershape_t *user_init(const char *str)
     }
     /* try to find size */
     bool saw_bb = false;
-    must_inline = false;
+    bool must_inline = false;
     while (fgets(line, sizeof(line), fp)) {
 	if (sscanf
 	    (line, "%%%%BoundingBox: %d %d %d %d", &lx, &ly, &ux, &uy) == 4) {
 	    saw_bb = true;
 	}
-	if ((line[0] != '%') && strstr(line,"read")) must_inline = true;
+	if (line[0] != '%' && strstr(line,"read")) must_inline = true;
 	if (saw_bb && must_inline) break;
     }
 
     if (saw_bb) {
-	us = GNEW(usershape_t);
+	us = gv_alloc(sizeof(usershape_t));
 	us->x = lx;
 	us->y = ly;
 	us->w = ux - lx;
@@ -73,7 +69,7 @@ static usershape_t *user_init(const char *str)
 	us->name = str;
 	us->macro_id = N_EPSF_files++;
 	fstat(fileno(fp), &statbuf);
-	contents = us->data = N_GNEW((size_t)statbuf.st_size + 1, char);
+	char *contents = us->data = gv_calloc((size_t)statbuf.st_size + 1, sizeof(char));
 	fseek(fp, 0, SEEK_SET);
 	size_t rc = fread(contents, (size_t)statbuf.st_size, 1, fp);
 	if (rc == 1) {
@@ -99,21 +95,19 @@ void epsf_init(node_t * n)
 {
     epsf_t *desc;
     const char *str;
-    usershape_t *us;
-    int dx, dy;
 
     if ((str = safefile(agget(n, "shapefile")))) {
-	us = user_init(str);
+	usershape_t *us = user_init(str);
 	if (!us)
 	    return;
-	dx = us->w;
-	dy = us->h;
+	int dx = us->w;
+	int dy = us->h;
 	ND_width(n) = PS2INCH(dx);
 	ND_height(n) = PS2INCH(dy);
-	ND_shape_info(n) = desc = NEW(epsf_t);
+	ND_shape_info(n) = desc = gv_alloc(sizeof(epsf_t));
 	desc->macro_id = us->macro_id;
-	desc->offset.x = -us->x - (dx) / 2;
-	desc->offset.y = -us->y - (dy) / 2;
+	desc->offset.x = -us->x - dx / 2;
+	desc->offset.y = -us->y - dy / 2;
     } else
 	agerr(AGWARN, "shapefile not set or not found for epsf node %s\n", agnameof(n));
 }
@@ -138,24 +132,23 @@ void epsf_free(node_t * n)
 void cat_libfile(GVJ_t * job, const char **arglib, const char **stdlib)
 {
     FILE *fp;
-    const char **s, *bp, *p;
-    int i;
+    const char *p;
     bool use_stdlib = true;
 
     /* check for empty string to turn off stdlib */
     if (arglib) {
-        for (i = 0; use_stdlib && ((p = arglib[i])); i++) {
+        for (int i = 0; use_stdlib && (p = arglib[i]); i++) {
             if (*p == '\0')
                 use_stdlib = false;
         }
     }
     if (use_stdlib)
-        for (s = stdlib; *s; s++) {
+        for (const char **s = stdlib; *s; s++) {
             gvputs(job, *s);
             gvputs(job, "\n");
         }
     if (arglib) {
-        for (i = 0; (p = arglib[i]) != 0; i++) {
+        for (int i = 0; (p = arglib[i]) != 0; i++) {
             if (*p == '\0')
                 continue;       /* ignore empty string */
             const char *safepath = safefile(p);    /* make sure filename is okay */
@@ -163,8 +156,14 @@ void cat_libfile(GVJ_t * job, const char **arglib, const char **stdlib)
 		agerr(AGWARN, "can't find library file %s\n", p);
 	    }
             else if ((fp = fopen(safepath, "r"))) {
-                while ((bp = Fgets(fp)))
-                    gvputs(job, bp);
+                while (true) {
+                    char bp[BUFSIZ] = {0};
+                    size_t r = fread(bp, 1, sizeof(bp), fp);
+                    gvwrite(job, bp, r);
+                    if (r < sizeof(bp)) {
+                      break;
+                    }
+                }
                 gvputs(job, "\n"); /* append a newline just in case */
 		fclose (fp);
             } else
@@ -213,11 +212,9 @@ void epsf_emit_body(GVJ_t *job, usershape_t *us)
 
 void epsf_define(GVJ_t *job)
 {
-    usershape_t *us;
-
     if (!EPSF_contents)
 	return;
-    for (us = dtfirst(EPSF_contents); us; us = dtnext(EPSF_contents, us)) {
+    for (usershape_t *us = dtfirst(EPSF_contents); us; us = dtnext(EPSF_contents, us)) {
 	if (us->must_inline)
 	    continue;
 	gvprintf(job, "/user_shape_%d {\n", us->macro_id);
@@ -261,7 +258,6 @@ charsetOf (char* s)
  */
 char *ps_string(char *ins, int chset)
 {
-    char *s;
     char *base;
     static agxbuf  xb;
     static int warned;
@@ -294,13 +290,10 @@ char *ps_string(char *ins, int chset)
 	}
     }
 
-    if (xb.buf == NULL)
-        agxbinit (&xb, 0, NULL);
-
     agxbputc (&xb, LPAREN);
-    s = base;
+    char *s = base;
     while (*s) {
-        if ((*s == LPAREN) || (*s == RPAREN) || (*s == '\\'))
+        if (*s == LPAREN || *s == RPAREN || *s == '\\')
             agxbputc (&xb, '\\');
         agxbputc (&xb, *s++);
     }
