@@ -27,11 +27,13 @@ extern "C" {
 #define _EXPARSE_H
 #endif
 
+#include <cgraph/agxbuf.h>
 #include <expr/exlib.h>
-#include <stddef.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
-#define exlex()		extoken_fn(expr.program)
+#define ex_lex()		extoken_fn(expr.program)
 
 #define ALLOCATE(p,x)	exalloc(p,sizeof(x))
 #define QUALIFY(r,s)	((r)&&(expr.program->disc->flags&EX_QUALIFY)?qualify(r,s):(s))
@@ -304,7 +306,7 @@ static Exnode_t *exstringOf(Expr_t * p, Exnode_t * x) {
 	    exerror("cannot convert %s to STRING", extypename(p, type));
 	if (x->op != CONSTANT) {
 	    if (!BUILTIN(type)) {
-		if ((*p->disc->stringof) (p, x, 1, p->disc) < 0) {
+		if (p->disc->stringof(p, x, 1) < 0) {
 		    exerror("cannot convert %s to STRING",
 			    extypename(p, type));
 		}
@@ -320,22 +322,18 @@ static Exnode_t *exstringOf(Expr_t * p, Exnode_t * x) {
 		}
 	    x = exnewnode(p, cvt, 0, STRING, x, 0);
 	} else if (!BUILTIN(type)) {
-	    if ((*p->disc->stringof) (p, x, 0, p->disc) < 0)
+	    if (p->disc->stringof(p, x, 0) < 0)
 		exerror("cannot convert constant %s to STRING",
 			extypename(p, x->type));
 	} else
 	    switch (type) {
 	    case FLOATING:
-		sfprintf(p->tmp, "%g", x->data.constant.value.floating);
 		x->data.constant.value.string =
-		    vmstrdup(p->vm, sfstruse(p->tmp));
+		  exprintf(p->vm, "%g", x->data.constant.value.floating);
 		break;
 	    case INTEGER:
-		sfprintf(p->tmp, "%I*d",
-			 sizeof(x->data.constant.value.integer),
-			 x->data.constant.value.integer);
 		x->data.constant.value.string =
-		    vmstrdup(p->vm, sfstruse(p->tmp));
+		  exprintf(p->vm, "%lld", (long long)x->data.constant.value.integer);
 		break;
 	    default:
 		exerror("internal error: %d: unknown type", type);
@@ -401,9 +399,7 @@ static Exnode_t *makeVar(Expr_t * prog, Exid_t * s, Exnode_t * idx,
 	if (!prog->disc->getf)
 	    exerror("%s: identifier references not supported", sym->name);
 	else if (expr.program->disc->reff)
-	    (*expr.program->disc->reff) (prog, nn,
-					 nn->data.variable.symbol, refs,
-					 NULL, EX_SCALAR, prog->disc);
+	    expr.program->disc->reff(prog, nn, nn->data.variable.symbol, refs);
 
 	return nn;
 }
@@ -436,7 +432,7 @@ static int	typecast[6][6] =
 char *extypename(Expr_t * p, int type) {
 	if (BUILTIN(type))
 	    return TYPENAME(type);
-	return (p->disc->typename) (p, type);
+	return p->disc->typename(type);
 }
 
 /* exnoncast:
@@ -469,8 +465,7 @@ excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
 		if (x->op != CONSTANT) {
 			Exid_t *sym = (xref ? xref->data.variable.symbol : NULL);
 			if (EXTERNAL(t2t)) {
-				int a = (arg ? arg : 1);
-		    	if ((*p->disc->convertf) (p, x, type, sym, a, p->disc) < 0) {
+		    	if (p->disc->convertf(x, type, 1) < 0) {
 					if (xref) {
 						if ((sym->lex == FUNCTION) && arg)
 							exerror ("%s: cannot use value of type %s as argument %d in function %s",
@@ -500,25 +495,25 @@ excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
 		case X2X:
 			if (xref && xref->op == ID)
 			{
-				if ((*p->disc->convertf)(p, x, type, xref->data.variable.symbol, arg, p->disc) < 0)
+				if (p->disc->convertf(x, type, arg) < 0)
 					exerror("%s: cannot cast constant %s to %s", xref->data.variable.symbol->name, extypename(p, x->type), extypename(p, type));
 			}
-			else if ((*p->disc->convertf)(p, x, type, NULL, arg, p->disc) < 0)
+			else if (p->disc->convertf(x, type, arg) < 0)
 				exerror("cannot cast constant %s to %s", extypename(p, x->type), extypename(p, type));
 			break;
 		case F2I:
 			x->data.constant.value.integer = x->data.constant.value.floating;
 			break;
 		case F2S:
-			sfprintf(p->tmp, "%g", x->data.constant.value.floating);
-			x->data.constant.value.string = exstash(p->tmp, p->vm);
+			x->data.constant.value.string =
+			  exprintf(p->vm, "%g", x->data.constant.value.floating);
 			break;
 		case I2F:
 			x->data.constant.value.floating = x->data.constant.value.integer;
 			break;
 		case I2S:
-			sfprintf(p->tmp, "%I*d", sizeof(x->data.constant.value.integer), x->data.constant.value.integer);
-			x->data.constant.value.string = exstash(p->tmp, p->vm);
+			x->data.constant.value.string =
+			  exprintf(p->vm, "%lld", (long long)x->data.constant.value.integer);
 			break;
 		case S2F:
 			s =  x->data.constant.value.string;
@@ -549,15 +544,19 @@ static Exid_t*
 qualify(Exref_t* ref, Exid_t* sym)
 {
 	Exid_t*	x;
-	char*			s;
 
 	while (ref->next)
 		ref = ref->next;
-	sfprintf(expr.program->tmp, "%s.%s", ref->symbol->name, sym->name);
-	s = exstash(expr.program->tmp, NULL);
+	size_t len = strlen(ref->symbol->name) + strlen(sym->name) + 2;
+	char *s = malloc(sizeof(char) * len);
+	if (s == NULL) {
+		exnospace();
+		return NULL;
+	}
+	snprintf(s, len, "%s.%s", ref->symbol->name, sym->name);
 	if (!(x = dtmatch(expr.program->symbols, s)))
 	{
-		if ((x = newof(0, Exid_t, 1, strlen(s) - EX_NAMELEN + 1)))
+		if ((x = calloc(1, sizeof(Exid_t) + strlen(s) - EX_NAMELEN + 1)))
 		{
 			memcpy(x, sym, sizeof(Exid_t) - EX_NAMELEN);
 			strcpy(x->name, s);
@@ -569,6 +568,7 @@ qualify(Exref_t* ref, Exid_t* sym)
 			x = sym;
 		}
 	}
+	free(s);
 	return x;
 }
 
@@ -618,7 +618,7 @@ preprint(Exnode_t* args)
 {
 	Print_t*	x;
 	char*		s;
-	int		c;
+	char		c;
 	int			t;
 	int			i;
 	int			n;
@@ -640,7 +640,7 @@ preprint(Exnode_t* args)
 	args = args->data.operand.right;
 	for (s = f; *s; s++)
 	{
-		sfputc(expr.program->tmp, *s);
+		agxbputc(&expr.program->tmp, *s);
 		if (*s == '%')
 		{
 			if (!*++s)
@@ -648,7 +648,7 @@ preprint(Exnode_t* args)
 			if (*s != '%')
 				break;
 			if (args)
-				sfputc(expr.program->tmp, '%');
+				agxbputc(&expr.program->tmp, '%');
 		}
 	}
 	x = 0;
@@ -692,7 +692,7 @@ preprint(Exnode_t* args)
 					n = 1;
 					for (;;)
 					{
-						sfputc(expr.program->tmp, c);
+						agxbputc(&expr.program->tmp, c);
 						switch (c = *s++)
 						{
 						case 0:
@@ -740,10 +740,10 @@ preprint(Exnode_t* args)
 						goto specified;
 					break;
 				}
-				sfputc(expr.program->tmp, c);
+				agxbputc(&expr.program->tmp, c);
 			}
 		specified:
-			sfputc(expr.program->tmp, c);
+			agxbputc(&expr.program->tmp, c);
 			for (e = s; *s; s++)
 			{
 				if (*s == '%')
@@ -760,7 +760,7 @@ preprint(Exnode_t* args)
 						break;
 					}
 				}
-				sfputc(expr.program->tmp, *s);
+				agxbputc(&expr.program->tmp, *s);
 			}
 			if (!args)
 			{
@@ -786,7 +786,7 @@ preprint(Exnode_t* args)
 				{
 					if (x->arg->op == CONSTANT && x->arg->data.constant.reference && expr.program->disc->convertf)
 					{
-						if ((*expr.program->disc->convertf)(expr.program, x->arg, STRING, x->arg->data.constant.reference, 0, expr.program->disc) < 0)
+						if (expr.program->disc->convertf(x->arg, STRING, 0) < 0)
 							exerror("cannot convert string format argument");
 						else x->arg->data.constant.value.string = vmstrdup(expr.program->vm, x->arg->data.constant.value.string);
 					}
@@ -799,7 +799,10 @@ preprint(Exnode_t* args)
 			}
 			args = args->data.operand.right;
 		}
-		x->format = exstash(expr.program->tmp, expr.program->vm);
+		x->format = vmstrdup(expr.program->vm, agxbuse(&expr.program->tmp));
+		if (x->format == NULL) {
+			x->format = exnospace();
+		}
 		if (!*s)
 			break;
 		f = s;
@@ -807,7 +810,7 @@ preprint(Exnode_t* args)
 	if (args)
 		exerror("too many format arguments");
  done:
-	sfstrseek(expr.program->tmp, 0, SEEK_SET);
+	agxbclear(&expr.program->tmp);
 	return p;
 }
 
@@ -815,39 +818,34 @@ preprint(Exnode_t* args)
  * push a new input stream and program
  */
 
-int
-expush(Expr_t* p, const char* name, int line, const char* sp, Sfio_t* fp)
-{
+int expush(Expr_t *p, const char *name, int line, FILE *fp) {
 	Exinput_t*	in;
 	char*		s;
 	char			buf[PATH_MAX];
 
-	if (!(in = newof(0, Exinput_t, 1, 0)))
+	if (!(in = calloc(1, sizeof(Exinput_t))))
 	{
 		exnospace();
 		return -1;
 	}
 	if (!p->input)
 		p->input = &expr.null;
-	if (!(in->bp = in->sp = (char*)sp))
+	in->bp = in->sp = NULL;
+	if ((in->fp = fp))
+		in->close = 0;
+	else if (name)
 	{
-		if ((in->fp = fp))
-			in->close = 0;
-		else if (name)
+		if (!(s = pathfind(name, p->disc->lib, p->disc->type, buf, sizeof(buf))) || !(in->fp = fopen(s, "r")))
 		{
-			if (!(s = pathfind(name, p->disc->lib, p->disc->type, buf, sizeof(buf))) || !(in->fp = sfopen(NULL, s, "r")))
-			{
-				exerror("%s: file not found", name);
-				in->bp = in->sp = "";
-			}
-			else
-			{
-				name = (const char*)vmstrdup(p->vm, s);
-				in->close = 1;
-			}
+			exerror("%s: file not found", name);
+			in->bp = in->sp = "";
+		}
+		else
+		{
+			name = vmstrdup(p->vm, s);
+			in->close = 1;
 		}
 	}
-	else in->fp = 0;
 	if (!(in->next = p->input)->next)
 	{
 		p->errors = 0;
@@ -895,7 +893,7 @@ expop(Expr_t* p)
 	else
 	{
 		if (p->errors && in->fp && p->linep != p->line)
-			while ((c = sfgetc(in->fp)) != EOF)
+			while ((c = getc(in->fp)) != EOF)
 				if (c == '\n')
 				{
 					error_info.line++;
@@ -905,9 +903,8 @@ expop(Expr_t* p)
 			error_info.line = in->line;
 	}
 	if (in->fp && in->close)
-		sfclose(in->fp);
-	if (in->pushback)
-		free(in->pushback);
+		fclose(in->fp);
+	free(in->pushback);
 	p->input = in->next;
 	free(in);
 	setcontext(p);
@@ -924,28 +921,17 @@ void exinit(void) {
 	memset (&expr, 0, sizeof(Exstate_t));
 }
 
-/*
- * compile the expression in [sf]p
- */
-
-int
-excomp(Expr_t* p, const char* name, int line, const char* sp, Sfio_t* fp)
-{
+int excomp(Expr_t *p, const char *name, int line, FILE *fp, char *prefix) {
 	Exid_t*	v;
 	int	eof;
 
-	p->more = 0;
 	eof = p->eof;
-	if (!sp && !fp)
-	{
-		if (!p->input)
-			return -1;
-	}
-	else if (expush(p, name, line, sp, fp))
+	if (expush(p, name, line, fp))
 		return -1;
-	else
-		p->input->unit = line >= 0;
-	exparse();
+	p->input->unit = line >= 0;
+	// insert prefix as pre-loaded pushback
+	p->input->pushback = p->input->pp = prefix;
+	ex_parse();
 	p->input->unit = 0;
 	expop(p);
 	p->eof = eof;
@@ -979,21 +965,19 @@ exclose(Expr_t* p, int all)
 			size_t i;
 			for (i = 3; i < elementsof(p->file); i++)
 				if (p->file[i])
-					sfclose(p->file[i]);
+					fclose(p->file[i]);
 			if (p->vm)
 				vmclose(p->vm);
 			if (p->ve)
 				vmclose(p->ve);
 			if (p->symbols)
 				dtclose(p->symbols);
-			if (p->tmp)
-				sfclose(p->tmp);
+			agxbfree(&p->tmp);
 			while ((in = p->input))
 			{
-				if (in->pushback)
-					free(in->pushback);
+				free(in->pushback);
 				if (in->fp && in->close)
-					sfclose(in->fp);
+					fclose(in->fp);
 				if ((p->input = in->next))
 					free(in);
 			}
@@ -1015,7 +999,7 @@ exclose(Expr_t* p, int all)
 static void
 checkBinary(Expr_t * p, Exnode_t * l, Exnode_t * ex, Exnode_t * r) 
 {
-	if ((*p->disc->binaryf) (p, l, ex, r, 1, p->disc) < 0) {
+	if (p->disc->binaryf(l, ex, r, 1) < 0) {
 	    if (r)
 		exerror
 		    ("cannot apply operator %s to expressions of types %s and %s",
@@ -1057,6 +1041,9 @@ static void checkName(Exid_t * id)
 static int
 cmpKey(Dt_t * d, Extype_t * key1, Extype_t * key2, Dtdisc_t * disc) 
 {
+	(void)d;
+	(void)disc;
+
 	if (key1->integer < key2->integer)
 	    return -1;
 	else if (key1->integer > key2->integer)
@@ -1068,7 +1055,7 @@ cmpKey(Dt_t * d, Extype_t * key1, Extype_t * key2, Dtdisc_t * disc)
 int
 exisAssign(Exnode_t * n) 
 {
-	return ((n->op == '=') && (n->subop == '='));
+	return n->op == '=' && n->subop == '=';
 }
 
 #endif
