@@ -85,8 +85,6 @@ int gvRender(GVC_t *gvc, graph_t *g, const char *format, FILE *out)
     int rc;
     GVJ_t *job;
 
-    g = g->root;
-
     /* create a job for the required format */
     bool r = gvjobs_output_langname(gvc, format);
     job = gvc->job;
@@ -117,8 +115,6 @@ int gvRenderFilename(GVC_t *gvc, graph_t *g, const char *format, const char *fil
     int rc;
     GVJ_t *job;
 
-    g = g->root;
-
     /* create a job for the required format */
     bool r = gvjobs_output_langname(gvc, format);
     job = gvc->job;
@@ -148,8 +144,6 @@ int gvRenderContext(GVC_t *gvc, graph_t *g, const char *format, void *context)
     int rc;
     GVJ_t *job;
 	
-    g = g->root;
-	
     /* create a job for the required format */
     bool r = gvjobs_output_langname(gvc, format);
     job = gvc->job;
@@ -182,8 +176,6 @@ int gvRenderData(GVC_t *gvc, graph_t *g, const char *format, char **result, unsi
     int rc;
     GVJ_t *job;
 
-    g = g->root;
-
     /* create a job for the required format */
     bool r = gvjobs_output_langname(gvc, format);
     job = gvc->job;
@@ -240,3 +232,11 @@ void gvAddLibrary(GVC_t *gvc, gvplugin_library_t *lib)
 char **gvcInfo(GVC_t* gvc) { return gvc->common.info; }
 char *gvcVersion(GVC_t* gvc) { return gvc->common.info[1]; }
 char *gvcBuildDate(GVC_t* gvc) { return gvc->common.info[2]; }
+
+/**
+ * @dir lib/gvc
+ * @brief Graphviz context library, API gvc.h
+ *
+ * [man 3 gvc](https://graphviz.org/pdf/gvc.3.pdf)
+ *
+ */
