@@ -9,6 +9,8 @@
  *************************************************************************/
 
 #include <math.h>
+#include <cgraph/alloc.h>
+#include <cgraph/exit.h>
 #include <neatogen/neato.h>
 #include <pathplan/pathutil.h>
 
@@ -53,9 +55,16 @@ typedef struct polygon polygon;
 	int number;
     } active_edge_list ;
     typedef struct {
-	int nvertices, npolygons, ninters;
+	int nvertices, ninters;
     } data ;
 
+static int sign(double v) {
+  if (v < 0)
+    return -1;
+  if (v > 0)
+    return 1;
+  return 0;
+}
 
 /* find the sign of the area of each of the triangles
     formed by adding a vertex of m to l  
@@ -72,9 +81,9 @@ static void sgnarea(vertex *l, vertex *m, int i[])
     g = after(m)->pos.x - a;
     h = after(m)->pos.y - b;
     t = c * f - d * e;
-    i[0] = t == 0 ? 0 : (t > 0 ? 1 : -1);
+    i[0] = sign(t);
     t = c * h - d * g;
-    i[1] = t == 0 ? 0 : (t > 0 ? 1 : -1);
+    i[1] = sign(t);
     i[2] = i[0] * i[1];
 }
 
@@ -256,7 +265,7 @@ static int find_intersection(vertex *l,
 #ifdef RECORD_INTERSECTS
     if (input->ninters >= MAXINTS) {
 	agerr(AGERR, "using too many intersections\n");
-	exit(1);
+	graphviz_exit(1);
     }
 
     ilist[input->ninters].firstv = l;
@@ -266,6 +275,9 @@ static int find_intersection(vertex *l,
     ilist[input->ninters].x = x;
     ilist[input->ninters].y = y;
     input->ninters++;
+#else
+    (void)ilist;
+    (void)input;
 #endif
     p.x = x;
     p.y = y;
@@ -299,13 +311,13 @@ find_ints(vertex vertex_list[], data *input, intersection ilist[]) {
     int i, j, k, found = 0;
     active_edge_list all;
     active_edge *new, *tempa;
-    vertex *pt1, *pt2, *templ, **pvertex;
+    vertex *pt1, *pt2, *templ;
 
     input->ninters = 0;
     all.first = all.final = 0;
     all.number = 0;
 
-    pvertex = N_GNEW(input->nvertices, vertex *);
+    vertex **pvertex = gv_calloc(input->nvertices, sizeof(vertex*));
 
     for (i = 0; i < input->nvertices; i++)
 	pvertex[i] = vertex_list + i;
@@ -330,7 +342,7 @@ find_ints(vertex vertex_list[], data *input, intersection ilist[]) {
 			goto finish;
 		}
 
-		new = GNEW(active_edge);
+		new = gv_alloc(sizeof(active_edge));
 		if (all.number == 0) {
 		    all.first = new;
 		    new->last = 0;
@@ -466,7 +478,6 @@ int Plegal_arrangement(Ppoly_t ** polys, int n_polys)
     }
 
     input.nvertices = nverts;
-    input.npolygons = n_polys;
 
     found = find_ints(vertex_list, &input, ilist);
     if (found < 0) {
