@@ -8,6 +8,7 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <cgraph/alloc.h>
 #include <float.h>
 #include <neatogen/multispline.h>
 #include <neatogen/delaunay.h>
@@ -60,25 +61,18 @@ static int cmpItem(Dt_t * d, int p1[], int p2[], Dtdisc_t * disc)
     (void)d;
     (void)disc;
 
-    if (p1[0] < p2[0])
-	return -1;
-    else if (p1[0] > p2[0])
-	return 1;
-    else if (p1[1] < p2[1])
-	return -1;
-    else if (p1[1] > p2[1])
-	return 1;
-    else
-	return 0;
+    if (p1[0] < p2[0]) return -1;
+    if (p1[0] > p2[0]) return 1;
+    if (p1[1] < p2[1]) return -1;
+    if (p1[1] > p2[1]) return 1;
+    return 0;
 }
 
 /* newItem:
  */
-static void *newItem(Dt_t * d, item * objp, Dtdisc_t * disc)
-{
-    item *newp = NEW(item);
+static void *newItem(item *objp, Dtdisc_t *disc) {
+    item *newp = gv_alloc(sizeof(item));
 
-    (void)d;
     (void)disc;
     newp->a[0] = objp->a[0];
     newp->a[1] = objp->a[1];
@@ -87,9 +81,7 @@ static void *newItem(Dt_t * d, item * objp, Dtdisc_t * disc)
     return newp;
 }
 
-static void freeItem(Dt_t * d, item * obj, Dtdisc_t * disc)
-{
-    (void)d;
+static void freeItem(item *obj, Dtdisc_t *disc) {
     (void)disc;
     free(obj);
 }
@@ -170,14 +162,18 @@ static int cmpIpair(Dt_t * d, int *p1, int *p2, Dtdisc_t * disc)
     (void)d;
     (void)disc;
 
-    return (*p1 - *p2);
+    if (*p1 < *p2) {
+        return -1;
+    }
+    if (*p1 > *p2) {
+        return 1;
+    }
+    return 0;
 }
 
-static void *newIpair(Dt_t * d, Ipair * objp, Dtdisc_t * disc)
-{
-    Ipair *newp = NEW(Ipair);
+static void *newIpair(Ipair *objp, Dtdisc_t *disc) {
+    Ipair *newp = gv_alloc(sizeof(Ipair));
 
-    (void)d;
     (void)disc;
     newp->i = objp->i;
     newp->j = objp->j;
@@ -185,9 +181,7 @@ static void *newIpair(Dt_t * d, Ipair * objp, Dtdisc_t * disc)
     return newp;
 }
 
-static void freeIpair(Dt_t * d, Ipair * obj, Dtdisc_t * disc)
-{
-    (void)d;
+static void freeIpair(Ipair *obj, Dtdisc_t *disc) {
     (void)disc;
     free(obj);
 }
@@ -232,7 +226,7 @@ static void mapTri(Dt_t * map, tri * tp)
 static tri *
 addTri(int i, int j, tri * oldp)
 {
-    tri *tp = NEW(tri);
+    tri *tp = gv_alloc(sizeof(tri));
     tp->v.i = i;
     tp->v.j = j;
     tp->nxttri = oldp;
@@ -260,9 +254,9 @@ static int raySeg(pointf v, pointf w, pointf a, pointf b)
     if (wa == wb)
 	return 0;
     if (wa == 0) {
-	return (wind(v, b, w) * wind(v, b, a) >= 0);
+	return wind(v, b, w) * wind(v, b, a) >= 0;
     } else {
-	return (wind(v, a, w) * wind(v, a, b) >= 0);
+	return wind(v, a, w) * wind(v, a, b) >= 0;
     }
 }
 
@@ -310,7 +304,7 @@ static int ctrlPtIdx(pointf v, Ppoly_t * polys)
     int i;
     for (i = 1; i < polys->pn; i++) {
 	w = polys->ps[i];
-	if ((w.x == v.x) && (w.y == v.y))
+	if (w.x == v.x && w.y == v.y)
 	    return i;
     }
     return -1;
@@ -331,8 +325,7 @@ static int ctrlPtIdx(pointf v, Ppoly_t * polys)
 static pointf *mkCtrlPts(int s, int mult, pointf prev, pointf v,
 			   pointf nxt, tripoly_t * trip)
 {
-    pointf *ps;
-    int idx = ctrlPtIdx(v, &(trip->poly));
+    int idx = ctrlPtIdx(v, &trip->poly);
     int i;
     double d, sep, theta, sinTheta, cosTheta;
     pointf q, w;
@@ -340,7 +333,7 @@ static pointf *mkCtrlPts(int s, int mult, pointf prev, pointf v,
     if (idx < 0)
 	return NULL;
 
-    ps = N_GNEW(mult, pointf);
+    pointf *ps = gv_calloc(mult, sizeof(pointf));
     theta = bisect(prev, v, nxt);
     sinTheta = sin(theta);
     cosTheta = cos(theta);
@@ -388,7 +381,7 @@ static pointf *mkCtrlPts(int s, int mult, pointf prev, pointf v,
  */
 
 typedef struct {
-    int ne;         /* no. of edges. */
+    size_t ne;      // no. of edges.
     int *edges;     /* indices of edges adjacent to node. */
     pointf ctr;     /* center of triangle. */
 } tnode;
@@ -401,8 +394,9 @@ typedef struct {
 
 typedef struct {
     tnode *nodes;
+    size_t nnodes; // number of nodes
     tedge *edges;
-    int nedges;    /* no. of edges; no. of nodes is stored in router_t */
+    int nedges; // number of edges
 } tgraph;
 
 struct router_s {
@@ -451,14 +445,10 @@ static boxf bbox(Ppoly_t** obsp, int npoly, int *np)
 	obs = *obsp++;
 	for (j = 0; j < obs->pn; j++) {
 	    p = obs->ps[j];
-	    if (p.x < bb.LL.x)
-		bb.LL.x = p.x;
-	    if (p.x > bb.UR.x)
-		bb.UR.x = p.x;
-	    if (p.y < bb.LL.y)
-		bb.LL.y = p.y;
-	    if (p.y > bb.UR.y)
-		bb.UR.y = p.y;
+	    bb.LL.x = fmin(bb.LL.x, p.x);
+	    bb.UR.x = fmax(bb.UR.x, p.x);
+	    bb.LL.y = fmin(bb.LL.y, p.y);
+	    bb.UR.y = fmax(bb.UR.y, p.y);
 	    cnt++;
 	}
     }
@@ -475,26 +465,11 @@ static boxf bbox(Ppoly_t** obsp, int npoly, int *np)
 
 static int *mkTriIndices(surface_t * sf)
 {
-    int *tris = N_GNEW(3 * sf->nfaces, int);
+    int *tris = gv_calloc(3 * sf->nfaces, sizeof(int));
     memcpy(tris, sf->faces, 3 * sf->nfaces * sizeof(int));
     return tris;
 }
 
-/* degT:
- * Given a pointer to an array of 3 integers, return
- * the number not equal to -1
- */
-static int degT(int *ip)
-{
-    ip++;
-    if (*ip++ == -1)
-	return 1;
-    if (*ip == -1)
-	return 2;
-    else
-	return 3;
-}
-
 /* sharedEdge:
  * Returns a pair of integer (x,y), x < y, where x and y are the
  * indices of the two vertices of the shared edge.
@@ -506,15 +481,15 @@ static ipair sharedEdge(int *p, int *q)
     p1 = *p;
     p2 = *(p + 1);
     if (p1 == *q) {
-	if ((p2 != *(q + 1)) && (p2 != *(q + 2))) {
+	if (p2 != *(q + 1) && p2 != *(q + 2)) {
 	    p2 = *(p + 2);
 	}
     } else if (p1 == *(q + 1)) {
-	if ((p2 != *q) && (p2 != *(q + 2))) {
+	if (p2 != *q && p2 != *(q + 2)) {
 	    p2 = *(p + 2);
 	}
     } else if (p1 == *(q + 2)) {
-	if ((p2 != *q) && (p2 != *(q + 1))) {
+	if (p2 != *q && p2 != *(q + 1)) {
 	    p2 = *(p + 2);
 	}
     } else {
@@ -532,11 +507,12 @@ static ipair sharedEdge(int *p, int *q)
 }
 
 /* addTriEdge:
- * Add an edge to g, with tail t, head h, distance d, and shared
+ * Add an edge to g, with tail t, head h, and shared
  * segment seg.
  */
-static void addTriEdge(tgraph * g, int t, int h, double d, ipair seg)
-{
+static void addTriEdge(tgraph *g, int t, int h, ipair seg) {
+    g->edges = gv_recalloc(g->edges, g->nedges, g->nedges + 1,
+                           sizeof(g->edges[0]));
     tedge *ep = g->edges + g->nedges;
     tnode *tp = g->nodes + t;
     tnode *hp = g->nodes + h;
@@ -546,7 +522,11 @@ static void addTriEdge(tgraph * g, int t, int h, double d, ipair seg)
     ep->dist = DIST(tp->ctr, hp->ctr);
     ep->seg = seg;
 
+    tp->edges = gv_recalloc(tp->edges, tp->ne, tp->ne + 1,
+                            sizeof(tp->edges[0]));
     tp->edges[tp->ne++] = g->nedges;
+    hp->edges = gv_recalloc(hp->edges, hp->ne, hp->ne + 1,
+                            sizeof(hp->edges[0]));
     hp->edges[hp->ne++] = g->nedges;
 
     g->nedges++;
@@ -554,7 +534,9 @@ static void addTriEdge(tgraph * g, int t, int h, double d, ipair seg)
 
 static void freeTriGraph(tgraph * tg)
 {
-    free(tg->nodes->edges);
+    for (size_t i = 0; i < tg->nnodes; ++i) {
+        free(tg->nodes[i].edges);
+    }
     free(tg->nodes);
     free(tg->edges);
     free(tg);
@@ -566,10 +548,8 @@ static void freeTriGraph(tgraph * tg)
  */
 static tgraph *mkTriGraph(surface_t * sf, int maxv, pointf * pts)
 {
-    tgraph *g;
     tnode *np;
     int j, i, ne = 0;
-    int *edgei;
     int *jp;
 
     /* ne is twice no. of edges */
@@ -577,43 +557,26 @@ static tgraph *mkTriGraph(surface_t * sf, int maxv, pointf * pts)
 	if (sf->neigh[i] != -1)
 	    ne++;
 
-    g = GNEW(tgraph);
+    tgraph *g = gv_alloc(sizeof(tgraph));
 
     /* plus 2 for nodes added as endpoints of an edge */
-    g->nodes = N_GNEW(sf->nfaces + 2, tnode);
-
-    /* allow 1 possible extra edge per triangle, plus 
-     * obstacles can have at most maxv triangles touching 
-     */
-    edgei = N_GNEW(sf->nfaces + ne + 2 * maxv, int);
-    g->edges = N_GNEW(ne/2 + 2 * maxv, tedge);
-    g->nedges = 0;
+    g->nnodes = sf->nfaces + 2;
+    g->nodes = gv_calloc(g->nnodes, sizeof(tnode));
 
     for (i = 0; i < sf->nfaces; i++) {
 	np = g->nodes + i;
-	np->ne = 0;
-	np->edges = edgei;
 	np->ctr = triCenter(pts, sf->faces + 3 * i);
-	edgei += degT(sf->neigh + 3 * i) + 1;
     }
-    /* initialize variable nodes */
-    np = g->nodes + i;
-    np->ne = 0;
-    np->edges = edgei;
-    np++;
-    np->ne = 0;
-    np->edges = edgei + maxv;
 
     for (i = 0; i < sf->nfaces; i++) {
 	np = g->nodes + i;
 	jp = sf->neigh + 3 * i;
         ne = 0;
-	while ((ne < 3) && ((j = *jp++) != -1)) {
+	while (ne < 3 && (j = *jp++) != -1) {
 	    if (i < j) {
-		double dist = DIST(np->ctr, (g->nodes + j)->ctr);
 		ipair seg =
 		    sharedEdge(sf->faces + 3 * i, sf->faces + 3 * j);
-		addTriEdge(g, i, j, dist, seg);
+		addTriEdge(g, i, j, seg);
 	    }
 	    ne++;
 	}
@@ -634,25 +597,21 @@ void freeRouter(router_t * rtr)
 
 router_t *mkRouter(Ppoly_t** obsp, int npoly)
 {
-    router_t *rtr = NEW(router_t);
+    router_t *rtr = gv_alloc(sizeof(router_t));
     Ppoly_t* obs;
     boxf bb;
-    pointf *pts;
     int npts;
     surface_t *sf;
-    int *segs;
-    double *x;
-    double *y;
     int maxv = 4; /* default max. no. of vertices in an obstacle; set below */
     /* points in obstacle i have indices obsi[i] through obsi[i+1]-1 in pts
      */
-    int *obsi = N_NEW(npoly + 1, int);
+    int *obsi = gv_calloc(npoly + 1, sizeof(int));
     int i, j, ix = 4, six = 0;
 
     bb = bbox(obsp, npoly, &npts);
     npts += 4;			/* 4 points of bounding box */
-    pts = N_GNEW(npts, pointf);	/* all points are stored in pts */
-    segs = N_GNEW(2 * npts, int);	/* indices of points forming segments */
+    pointf *pts = gv_calloc(npts, sizeof(pointf)); // all points are stored in pts
+    int *segs = gv_calloc(2 * npts, sizeof(int)); // indices of points forming segments
 
     /* store bounding box in CCW order */
     pts[0] = bb.LL;
@@ -687,8 +646,8 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
     obsi[i] = ix;
 
     /* copy points into coordinate arrays */
-    x = N_GNEW(npts, double);
-    y = N_GNEW(npts, double);
+    double *x = gv_calloc(npts, sizeof(double));
+    double *y = gv_calloc(npts, sizeof(double));
     for (i = 0; i < npts; i++) {
 	x[i] = pts[i].x;
 	y[i] = pts[i].y;
@@ -714,31 +673,19 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
  * Finish edge generation, clipping to nodes and adding arrowhead
  * if necessary, and adding edge labels
  */
-static void finishEdge(edge_t* e, Ppoly_t spl, int flip, pointf p, pointf q) {
-    int j;
-    pointf *spline = N_GNEW(spl.pn, pointf);
-    pointf p1, q1;
-
+static void finishEdge(edge_t* e, Ppoly_t spl, int flip) {
     if (flip) {
-	for (j = 0; j < spl.pn; j++) {
-	    spline[spl.pn - 1 - j] = spl.ps[j];
+	for (int j = 0; j < spl.pn / 2; j++) {
+	    pointf tmp = spl.ps[spl.pn - 1 - j];
+	    spl.ps[spl.pn - 1 - j] = spl.ps[j];
+	    spl.ps[j] = tmp;
 	}
-	p1 = q;
-	q1 = p;
-    }
-    else {
-	for (j = 0; j < spl.pn; j++) {
-	    spline[j] = spl.ps[j];
-	}
-	p1 = p;
-	q1 = q;
     }
     if (Verbose > 1)
 	fprintf(stderr, "spline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
-    clip_and_install(e, aghead(e), spline, spl.pn, &sinfo);
-    free(spline);
+    clip_and_install(e, aghead(e), spl.ps, spl.pn, &sinfo);
 
-    addEdgeLabels(e, p1, q1);
+    addEdgeLabels(e);
 }
 
 #define EQPT(p,q) (((p).x==(q).x)&&((p).y==(q).y))
@@ -757,9 +704,7 @@ static void finishEdge(edge_t* e, Ppoly_t spl, int flip, pointf p, pointf q) {
  *
  * Otherwise, return p unchanged.
  */
-static Ppoint_t
-tweakEnd (Ppoly_t poly, int s, Ppolyline_t pl, Ppoint_t q)
-{
+static Ppoint_t tweakEnd (Ppoly_t poly, int s, Ppoint_t q) {
     Ppoint_t prv, nxt, p;
 
     p = poly.ps[s];
@@ -783,8 +728,8 @@ tweakEnd (Ppoly_t poly, int s, Ppolyline_t pl, Ppoint_t q)
 static void
 tweakPath (Ppoly_t poly, int s, int t, Ppolyline_t pl)
 {
-    pl.ps[0] = tweakEnd (poly, s, pl, pl.ps[1]);
-    pl.ps[pl.pn-1] = tweakEnd (poly, t, pl, pl.ps[pl.pn-2]);
+    pl.ps[0] = tweakEnd (poly, s, pl.ps[1]);
+    pl.ps[pl.pn-1] = tweakEnd (poly, t, pl.ps[pl.pn-2]);
 }
 
 
@@ -802,7 +747,7 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
     Ppolyline_t pl, spl;
     int i, j;
     Ppolyline_t mmpl;
-    Pedge_t *medges = N_GNEW(trip->poly.pn, Pedge_t);
+    Pedge_t *medges = NULL;
     int pn;
     int mult = ED_count(e);
     node_t* head = aghead(e);
@@ -826,8 +771,9 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
     evs[0].x = evs[0].y = 0;
     evs[1].x = evs[1].y = 0;
 
-    if ((mult == 1) || Concentrate) {
+    if (mult == 1 || Concentrate) {
 	poly = trip->poly;
+	medges = gv_calloc(poly.pn, sizeof(Pedge_t));
 	for (j = 0; j < poly.pn; j++) {
 	    medges[j].a = poly.ps[j];
 	    medges[j].b = poly.ps[(j + 1) % poly.pn];
@@ -838,7 +784,7 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 	    rv = 1;
 	    goto finish;
 	}
-	finishEdge(e, spl, aghead(e) != head, eps[0], eps[1]);
+	finishEdge(e, spl, aghead(e) != head);
 	free(medges);
 
 	return 0;
@@ -846,7 +792,7 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
     
     pn = 2 * (pl.pn - 1);
 
-    cpts = N_NEW(pl.pn - 2, pointf *);
+    cpts = gv_calloc(pl.pn - 2, sizeof(pointf *));
     for (i = 0; i < pl.pn - 2; i++) {
 	cpts[i] =
 	    mkCtrlPts(t, mult+1, pl.ps[i], pl.ps[i + 1], pl.ps[i + 2], trip);
@@ -857,7 +803,7 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 	}
     }
 
-    poly.ps = N_GNEW(pn, pointf);
+    poly.ps = gv_calloc(pn, sizeof(pointf));
     poly.pn = pn;
 
     for (i = 0; i < mult; i++) {
@@ -879,6 +825,7 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 	    make_polyline (mmpl, &spl);
 	}
 	else {
+	    medges = gv_calloc(poly.pn, sizeof(Pedge_t));
 	    for (j = 0; j < poly.pn; j++) {
 		medges[j].a = poly.ps[j];
 		medges[j].b = poly.ps[(j + 1) % poly.pn];
@@ -891,7 +838,7 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 		goto finish;
 	    }
 	}
-	finishEdge(e, spl, aghead(e) != head, eps[0], eps[1]);
+	finishEdge(e, spl, aghead(e) != head);
 
 	e = ED_to_virt(e);
     }
@@ -915,7 +862,7 @@ finish :
 static int
 inCone (pointf a, pointf b, pointf c, pointf q)
 {
-    return ((area2(q,a,b) >= NSMALL) && (area2(q,b,c) >= NSMALL));
+  return area2(q,a,b) >= NSMALL && area2(q,b,c) >= NSMALL;
 }
 
 static pointf north = {0, 1};
@@ -941,7 +888,6 @@ static void addEndpoint(router_t * rtr, pointf p, node_t* v, int v_id, int sides
     int endi = rtr->obs[obs_id + 1];
     pointf* pts = rtr->ps;
     int i, t;
-    double d;
     pointf vr, v0, v1;
 
     switch (sides) {
@@ -1004,8 +950,7 @@ static void addEndpoint(router_t * rtr, pointf p, node_t* v, int v_id, int sides
 	t = findMap(rtr->trimap, seg.i, seg.j);
 	if (sides && !inCone (v0, p, v1, pts[seg.i]) && !inCone (v0, p, v1, pts[seg.j]) && !raySeg(p,vr,pts[seg.i],pts[seg.j]))
 	    continue;
-	d = DIST(p, (rtr->tg->nodes + t)->ctr);
-	addTriEdge(rtr->tg, v_id, t, d, seg);
+	addTriEdge(rtr->tg, v_id, t, seg);
     }
 }
 
@@ -1023,10 +968,10 @@ static ipair edgeToSeg(tgraph * tg, int i, int j)
     tnode *np = tg->nodes + i;
     tedge *ep;
 
-    for (i = 0; i < np->ne; i++) {
-	ep = tg->edges + np->edges[i];
-	if ((ep->t == j) || (ep->h == j))
-	    return (ep->seg);
+    for (size_t k = 0; k < np->ne; k++) {
+	ep = tg->edges + np->edges[k];
+	if (ep->t == j || ep->h == j)
+	    return ep->seg;
     }
 
     assert(0);
@@ -1076,23 +1021,19 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
     int nxt;
     ipair p;
     int nt = 0;
-    side_t *side1;
-    side_t *side2;
     int i, idx;
     int cnt1 = 0;
     int cnt2 = 0;
     pointf *pts;
-    pointf *pps;
     /* maps vertex index used in router_t to vertex index used in tripoly */
     Dt_t *vmap;
-    tri **trim;
 
     /* count number of triangles in path */
     for (nxt = dad[t]; nxt != s; nxt = dad[nxt])
 	nt++;
 
-    side1 = N_NEW(nt + 4, side_t);
-    side2 = N_NEW(nt + 4, side_t);
+    side_t *side1 = gv_calloc(nt + 4, sizeof(side_t));
+    side_t *side2 = gv_calloc(nt + 4, sizeof(side_t));
 
     nxt = dad[t];
     p = edgeToSeg(rtr->tg, nxt, t);
@@ -1148,8 +1089,8 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
     vmap = dtopen(&ipairdisc, Dtoset);
     vmapAdd(vmap, -1, 0);
     vmapAdd(vmap, -2, cnt1 + 1);
-    pps = pts = N_GNEW(nt + 4, pointf);
-    trim = N_NEW(nt + 4, tri *);
+    pointf *pps = pts = gv_calloc(nt + 4, sizeof(pointf));
+    tri **trim = gv_calloc(nt + 4, sizeof(tri*));
     *pps++ = p_t;
     idx = 1;
     for (i = 0; i < cnt1; i++) {
@@ -1169,7 +1110,7 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
 	mapTri(vmap, trim[i]);
     }
 
-    ps = NEW(tripoly_t);
+    ps = gv_alloc(sizeof(tripoly_t));
     ps->poly.pn = nt + 4;  /* nt triangles gives nt+2 points plus s and t */
     ps->poly.ps = pts;
     ps->triMap = trim;
@@ -1184,14 +1125,13 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
 /* resetGraph:
  * Remove edges and nodes added for current edge routing
  */
-static void resetGraph(tgraph * g, int ncnt, int ecnt)
-{
+static void resetGraph(tgraph *g, int ncnt, int ecnt,
+                       size_t *original_edge_count) {
     int i;
     tnode *np = g->nodes;
     g->nedges = ecnt;
     for (i = 0; i < ncnt; i++) {
-	if (np->edges + np->ne == (np + 1)->edges)
-	    np->ne--;
+	np->ne = original_edge_count[i];
 	np++;
     }
 }
@@ -1229,11 +1169,11 @@ typedef struct {
 static int *
 triPath(tgraph * g, int n, int v0, int v1, PQ * pq)
 {
-    int i, j, adjn;
+    int i, adjn;
     double d;
     tnode *np;
     tedge *e;
-    int *dad = N_NEW(n, int);
+    int *dad = gv_calloc(n, sizeof(int));
 
     for (i = 0; i < pq->PQsize; i++)
 	N_VAL(pq, i) = UNSEEN;
@@ -1249,7 +1189,7 @@ triPath(tgraph * g, int n, int v0, int v1, PQ * pq)
 	if (i == v1)
 	    break;
 	np = g->nodes + i;
-	for (j = 0; j < np->ne; j++) {
+	for (size_t j = 0; j < np->ne; j++) {
 	    e = g->edges + np->edges[j];
 	    if (e->t == i)
 		adjn = e->h;
@@ -1289,18 +1229,23 @@ int makeMultiSpline(edge_t* e, router_t * rtr, int doPolyline) {
     int h_id = rtr->tn + 1;
     int ecnt = rtr->tg->nedges;
     PPQ pq;
-    PQTYPE *idxs;
-    PQVTYPE *vals;
     int ret;
 
+    // record the number of edges in each node, so we can drop the added ones
+    // later
+    size_t *original_edge_count = gv_calloc(rtr->tg->nnodes,
+                                            sizeof(original_edge_count[0]));
+    for (size_t i = 0; i < rtr->tg->nnodes; ++i)
+        original_edge_count[i] = rtr->tg->nodes[i].ne;
+
 	/* Add endpoints to triangle graph */
     addEndpoint(rtr, t_p, t, t_id, ED_tail_port(e).side);
     addEndpoint(rtr, h_p, h, h_id, ED_head_port(e).side);
 
 	/* Initialize priority queue */
     PQgen(&pq.pq, rtr->tn + 2, -1);
-    idxs = N_GNEW(pq.pq.PQsize + 1, PQTYPE);
-    vals = N_GNEW(pq.pq.PQsize + 1, PQVTYPE);
+    PQTYPE *idxs = gv_calloc(pq.pq.PQsize + 1, sizeof(PQTYPE));
+    PQVTYPE *vals = gv_calloc(pq.pq.PQsize + 1, sizeof(PQVTYPE));
     vals[0] = 0;
     pq.vals = vals + 1;
     pq.idxs = idxs + 1;
@@ -1323,6 +1268,7 @@ int makeMultiSpline(edge_t* e, router_t * rtr, int doPolyline) {
     }
     else ret = -1;
 
-    resetGraph(rtr->tg, rtr->tn, ecnt);
+    resetGraph(rtr->tg, rtr->tn, ecnt, original_edge_count);
+    free(original_edge_count);
     return ret;
 }
