@@ -1,3 +1,10 @@
+/**
+ * @file
+ * @brief API neatogen/neatoprocs.h:
+ * @ref neato_init_node, @ref user_pos, @ref neato_cleanup,
+ * @ref init_nop, @ref setSeed, @ref checkStart, @ref neato_layout
+ */
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
@@ -26,7 +33,11 @@
 #include <neatogen/kkutils.h>
 #include <common/pointset.h>
 #include <neatogen/sgd.h>
+#include <cgraph/alloc.h>
+#include <cgraph/bitarray.h>
+#include <cgraph/startswith.h>
 #include <cgraph/strcasecmp.h>
+#include <stdbool.h>
 
 #ifndef HAVE_SRAND48
 #define srand48 srand
@@ -40,7 +51,7 @@ static char *cc_pfx = "_neato_cc";
 
 void neato_init_node(node_t * n)
 {
-    agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
+    agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);	//node custom data
     common_init_node(n);
     ND_pos(n) = N_NEW(GD_ndim(agraphof(n)), double);
     gv_nodesize(n, GD_flip(agraphof(n)));
@@ -48,7 +59,7 @@ void neato_init_node(node_t * n)
 
 static void neato_init_edge(edge_t * e)
 {
-    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//node custom data
+    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);	//node custom data
     common_init_edge(e);
     ED_factor(e) = late_double(e, E_weight, 1.0, 1.0);
 }
@@ -133,8 +144,6 @@ static void neato_cleanup_graph(graph_t * g)
 	free_scan_graph(g);
 	free(GD_clust(g));
     }
-    if (g != agroot(g))
-        agclean(g, AGRAPH , "Agraphinfo_t");
 }
 
 void neato_cleanup(graph_t * g)
@@ -174,8 +183,8 @@ static void set_label(void* obj, textlabel_t * l, char *name)
     char *lp;
     lp = agget(obj, name);
     if (lp && sscanf(lp, "%lf,%lf", &x, &y) == 2) {
-	l->pos = pointfof(x, y);
-	l->set = TRUE;
+	l->pos = (pointf){x, y};
+	l->set = true;
     }
 }
 
@@ -187,8 +196,8 @@ static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
      /* array of arrays of node indices in each cluster */
     int **cs,*cn;
     int i,j,nclusters=0;
-    boolean* assigned = N_NEW(agnnodes(g), boolean);
-    cluster_data *cdata = GNEW(cluster_data);
+    bitarray_t assigned = bitarray_new(agnnodes(g));
+    cluster_data *cdata = gv_alloc(sizeof(cluster_data));
 
     cdata->ntoplevel = agnnodes(g);
     for (subg = agfstsubg(mastergraph); subg; subg = agnxtsubg(subg)) {
@@ -198,8 +207,8 @@ static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
     }
     cdata->nvars=0;
     cdata->nclusters = nclusters;
-    cs = cdata->clusters = N_GNEW(nclusters,int*);
-    cn = cdata->clustersizes = N_GNEW(nclusters,int);
+    cs = cdata->clusters = gv_calloc(nclusters, sizeof(int*));
+    cn = cdata->clustersizes = gv_calloc(nclusters, sizeof(int));
     for (subg = agfstsubg(mastergraph); subg; subg = agnxtsubg(subg)) {
         /* clusters are processed by separate calls to ordered_edges */
         if (!strncmp(agnameof(subg), "cluster", 7)) {
@@ -207,7 +216,7 @@ static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
 
             *cn = agnnodes(subg);
             cdata->nvars += *cn;
-            c = *cs++ = N_GNEW(*cn++,int);
+            c = *cs++ = gv_calloc(*cn++, sizeof(int));
             for (n = agfstnode(subg); n; n = agnxtnode(subg, n)) {
                 node_t *gn;
                 int ind = 0;
@@ -216,20 +225,20 @@ static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
                     ind++;
                 }
                 *c++=ind;
-                assigned[ind]=TRUE;
+                bitarray_set(&assigned, ind, true);
                 cdata->ntoplevel--;
             }
         }
     }
-    cdata->bb=N_GNEW(cdata->nclusters,boxf);
-    cdata->toplevel=N_GNEW(cdata->ntoplevel,int);
+    cdata->bb = gv_calloc(cdata->nclusters, sizeof(boxf));
+    cdata->toplevel = gv_calloc(cdata->ntoplevel, sizeof(int));
     for(i=j=0;i<agnnodes(g);i++) {
-        if(!assigned[i]) {
+        if(!bitarray_get(assigned, i)) {
             cdata->toplevel[j++]=i;
         }
     }
     assert(cdata->ntoplevel==agnnodes(g)-cdata->nvars);
-    free (assigned);
+    bitarray_reset(&assigned);
     return cdata;
 }
 
@@ -261,13 +270,13 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
     pointf sp = { 0, 0 }, ep = { 0, 0};
     bezier *newspl;
     int more = 1;
-    int stype, etype;
-    static boolean warned;
+    static bool warned;
 
     pos = agxget(e, E_pos);
     if (*pos == '\0')
 	return 0;
 
+    uint32_t stype, etype;
     arrow_flags(e, &stype, &etype);
     do {
 	/* check for s head */
@@ -293,7 +302,7 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	if (n < 4 || n % 3 != 1) {
 	    gv_free_splines(e);
 	    if (!warned) {
-		warned = 1;
+		warned = true;
 		agerr(AGWARN, "pos attribute for edge (%s,%s) doesn't have 3n+1 points\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    }
 	    return 0;
@@ -304,7 +313,7 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	    i = sscanf(pos, "%lf,%lf%n", &x, &y, &nc);
 	    if (i < 2) {
 		if (!warned) {
-		    warned = 1;
+		    warned = true;
 		    agerr(AGWARN, "syntax error in pos attribute for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 		}
 		free(ps);
@@ -317,7 +326,7 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	    pp++;
 	    n--;
 	}
- 	while (isspace(*pos)) pos++;
+ 	while (isspace((int)*pos)) pos++;
 	if (*pos == '\0')
 	    more = 0;
 	else
@@ -446,7 +455,8 @@ static void add_cluster(Agraph_t * g, Agraph_t * subg)
 {
     int cno;
     cno = ++(GD_n_cluster(g));
-    GD_clust(g) = ZALLOC(cno + 1, GD_clust(g), graph_t *, GD_n_cluster(g));
+    GD_clust(g) = gv_recalloc(GD_clust(g), GD_n_cluster(g), cno + 1,
+                              sizeof(graph_t*));
     GD_clust(g)[cno] = subg;
     do_graph_label(subg);
 }
@@ -467,7 +477,7 @@ dfs(Agraph_t * subg, Agraph_t * parentg, attrsym_t * G_lp, attrsym_t * G_bb)
     boxf bb;
 
     if (!strncmp(agnameof(subg), "cluster", 7) && chkBB(subg, G_bb, &bb)) {
-	agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+	agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	GD_bb(subg) = bb;
 	add_cluster(parentg, subg);
 	nop_init_graphs(subg, G_lp, G_bb);
@@ -494,8 +504,8 @@ nop_init_graphs(Agraph_t * g, attrsym_t * G_lp, attrsym_t * G_bb)
     if (GD_label(g) && G_lp) {
 	s = agxget(g, G_lp);
 	if (sscanf(s, "%lf,%lf", &x, &y) == 2) {
-	    GD_label(g)->pos = pointfof(x, y);
-	    GD_label(g)->set = TRUE;
+	    GD_label(g)->pos = (pointf){x, y};
+	    GD_label(g)->set = true;
 	}
     }
 
@@ -530,7 +540,7 @@ int init_nop(Agraph_t * g, int adjust)
     attrsym_t *G_bb = agfindgraphattr(g, "bb");
     int didAdjust = 0;  /* Have nodes been moved? */
     int haveBackground;
-    boolean translate = !mapBool(agget(g, "notranslate"), FALSE);
+    bool translate = !mapBool(agget(g, "notranslate"), false);
 
     /* If G_bb not defined, define it */
     if (!G_bb)
@@ -560,7 +570,7 @@ int init_nop(Agraph_t * g, int adjust)
 	didAdjust = adjustNodes(g);
 
     if (didAdjust) {
-	if (GD_label(g)) GD_label(g)->set = FALSE;
+	if (GD_label(g)) GD_label(g)->set = false;
 /* FIX:
  *   - if nodes are moved, clusters are no longer valid.
  */
@@ -584,7 +594,7 @@ int init_nop(Agraph_t * g, int adjust)
 	}
     }
     else {
-	boolean didShift;
+	bool didShift;
 	if (translate && !haveBackground && (GD_bb(g).LL.x != 0||GD_bb(g).LL.y != 0))
 	    neato_translate (g);
 	didShift = neato_set_aspect(g);
@@ -594,7 +604,7 @@ int init_nop(Agraph_t * g, int adjust)
 	    posEdges = NoEdges;
 	}
 	if (posEdges != AllEdges)
-	    spline_edges0(g, FALSE);   /* add edges */
+	    spline_edges0(g, false);   /* add edges */
 	else
 	    State = GVSPLINES;
     }
@@ -606,7 +616,7 @@ static void neato_init_graph (Agraph_t * g)
 {
     int outdim;
 
-    setEdgeType (g, ET_LINE);
+    setEdgeType (g, EDGETYPE_LINE);
     outdim = late_int(g, agfindgraphattr(g, "dimen"), 2, 2);
     GD_ndim(agroot(g)) = late_int(g, agfindgraphattr(g, "dim"), outdim, 2);
     Ndim = GD_ndim(g->root) = MIN(GD_ndim(g->root), MAXDIM);
@@ -707,7 +717,7 @@ dfsCycle (vtx_data* graph, int i,int mode, node_t* nodes[])
 
     np = nodes[i];
     ND_mark(np) = TRUE;
-    ND_onstack(np) = TRUE;
+    ND_onstack(np) = true;
     for (e = 1; e < graph[i].nedges; e++) {
 	if (graph[i].edists[e] == 1.0) continue;  /* in edge */
 	j = graph[i].edges[e];
@@ -718,10 +728,10 @@ dfsCycle (vtx_data* graph, int i,int mode, node_t* nodes[])
             assert (f < graph[j].nedges);
             graph[j].edists[f] = -1.0;
         }
-	else if (ND_mark(hp) == FALSE) dfsCycle(graph, j, mode, nodes);
+	else if (!ND_mark(hp)) dfsCycle(graph, j, mode, nodes);
 
     }
-    ND_onstack(np) = FALSE;
+    ND_onstack(np) = false;
 }
 
 /* acyclic:
@@ -736,7 +746,7 @@ acyclic (vtx_data* graph, int nv, int mode, node_t* nodes[])
     for (i = 0; i < nv; i++) {
 	np = nodes[i];
 	ND_mark(np) = FALSE;
-	ND_onstack(np) = FALSE;
+	ND_onstack(np) = false;
     }
     for (i = 0; i < nv; i++) {
 	if (ND_mark(nodes[i])) continue;
@@ -766,10 +776,7 @@ acyclic (vtx_data* graph, int nv, int mode, node_t* nodes[])
  */
 static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int model, node_t*** nodedata)
 {
-    vtx_data *graph;
-    node_t** nodes;
     int ne = agnedges(g);	/* upper bound */
-    int *edges;
     float *ewgts = NULL;
     node_t *np;
     edge_t *ep;
@@ -777,35 +784,29 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 #ifdef DIGCOLA
     float *edists = NULL;
 #endif
-    attrsym_t *haveLen;
-    int haveWt;
-    int haveDir;
     PointMap *ps = newPM();
     int i, i_nedges, idx;
 
     /* lengths and weights unused in reweight model */
-    if (model == MODEL_SUBSET) {
-	haveLen = FALSE;
-	haveWt = FALSE;
-    } else {
-	haveLen = agattr(g, AGEDGE, "len", 0) ;
+    bool haveLen = false;
+    bool haveWt = false;
+    if (model != MODEL_SUBSET) {
+	haveLen = agattr(g, AGEDGE, "len", 0) != NULL;
 	haveWt = E_weight != 0;
     }
-    if (mode == MODE_HIER || mode == MODE_IPSEP)
-	haveDir = TRUE;
-    else
-	haveDir = FALSE;
+    bool haveDir = mode == MODE_HIER || mode == MODE_IPSEP;
 
-    graph = N_GNEW(nv, vtx_data);
-    nodes = N_GNEW(nv, node_t*);
-    edges = N_GNEW(2 * ne + nv, int);	/* reserve space for self loops */
+    vtx_data *graph = gv_calloc(nv, sizeof(vtx_data));
+    node_t** nodes = gv_calloc(nv, sizeof(node_t*));
+    const size_t edges_size = (size_t)(2 * ne + nv);
+    int *edges = gv_calloc(edges_size, sizeof(int)); // reserve space for self loops
     if (haveLen || haveDir)
-	ewgts = N_GNEW(2 * ne + nv, float);
+	ewgts = gv_calloc(edges_size, sizeof(float));
     if (haveWt)
-	eweights = N_GNEW(2 * ne + nv, float);
+	eweights = gv_calloc(edges_size, sizeof(float));
 #ifdef DIGCOLA
     if (haveDir)
-	edists = N_GNEW(2*ne+nv,float);
+	edists = gv_calloc(edges_size, sizeof(float));
 #endif
 
     i = 0;
@@ -859,7 +860,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 #ifdef DIGCOLA
                 if (haveDir) {
                     char *s = agget(ep,"dir");
-                    if(s&&!strncmp(s,"none",4)) {
+                    if(s && startswith(s, "none")) {
                         *edists++ = 0;
                     } else {
                         *edists++ = np == aghead(ep) ? 1.0 : -1.0;
@@ -872,9 +873,6 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 
 	graph[i].nedges = i_nedges;
 	graph[i].edges[0] = i;
-#ifdef USE_STYLES
-	graph[i].styles = NULL;
-#endif
 	i++;
     }
 #ifdef DIGCOLA
@@ -888,11 +886,11 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 
     /* If necessary, release extra memory. */
     if (ne != agnedges(g)) {
-	edges = RALLOC(2 * ne + nv, graph[0].edges, int);
+	edges = gv_recalloc(graph[0].edges, edges_size, 2 * ne + nv, sizeof(int));
 	if (haveLen)
-	    ewgts = RALLOC(2 * ne + nv, graph[0].ewgts, float);
+	    ewgts = gv_recalloc(graph[0].ewgts, edges_size, 2 * ne + nv, sizeof(float));
 	if (haveWt)
-	    eweights = RALLOC(2 * ne + nv, graph[0].eweights, float);
+	    eweights = gv_recalloc(graph[0].eweights, edges_size, 2 * ne + nv, sizeof(float));
 
 	for (i = 0; i < nv; i++) {
 	    int sz = graph[i].nedges;
@@ -1159,7 +1157,7 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
     if (mode != MODE_MAJOR) {
         double lgap = late_double(g, agfindgraphattr(g, "levelsgap"), 0.0, -MAXDOUBLE);
         if (mode == MODE_HIER) {
-            rv = stress_majorization_with_hierarchy(gp, nv, ne, coords, nodes, Ndim,
+            rv = stress_majorization_with_hierarchy(gp, nv, coords, nodes, Ndim,
                        opts, model, MaxIter, lgap);
         }
 #ifdef IPSEPCOLA
@@ -1170,9 +1168,6 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
 	    cluster_data *cs = cluster_map(mg,g);
 	    nsize = N_GNEW(nv, pointf);
             opt.edge_gap = lgap;
-#ifdef MOSEK
-            opt.mosek = 0;
-#endif /* MOSEK */
             opt.nsize = nsize;
             opt.clusters = cs;
             str = agget(g, "diredgeconstraints");
@@ -1196,14 +1191,6 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
                     fprintf(stderr,"Removing overlaps as postprocess...\n");
             }
             else opt.noverlap = 0;
-#ifdef MOSEK
-            str = agget(g, "mosek");
-            if(str && !strncmp(str,"true",4)) {
-                opt.mosek = 1;
-                if(Verbose)
-                    fprintf(stderr,"Using Mosek for constraint optimization...\n");
-            }
-#endif /* MOSEK */
 	    margin = sepFactor (g);
  	    /* Multiply by 2 since opt.gap is the gap size, not the margin */
 	    if (margin.doAdd) {
@@ -1232,7 +1219,7 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
 	    fprintf (stderr, "\n");
 	    dumpOpts (&opt, nv);
 #endif
-            rv = stress_majorization_cola(gp, nv, ne, coords, nodes, Ndim, model, MaxIter, &opt);
+            rv = stress_majorization_cola(gp, nv, coords, nodes, Ndim, model, MaxIter, &opt);
 	    freeClusterData(cs);
 	    free (nsize);
         }
@@ -1240,7 +1227,7 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
     }
     else
 #endif
-	rv = stress_majorization_kD_mkernel(gp, nv, ne, coords, nodes, Ndim, opts, model, MaxIter);
+	rv = stress_majorization_kD_mkernel(gp, nv, coords, nodes, Ndim, opts, model, MaxIter);
 
     if (rv < 0) {
 	agerr(AGPREV, "layout aborted\n");
@@ -1384,7 +1371,7 @@ addCluster (graph_t* g)
     graph_t *subg;
     for (subg = agfstsubg(agroot(g)); subg; subg = agnxtsubg(subg)) {
 	if (!strncmp(agnameof(subg), "cluster", 7)) {
-	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	    add_cluster(g, subg);
 	    compute_bb(subg);
 	}
@@ -1399,7 +1386,7 @@ addCluster (graph_t* g)
 static void doEdges(Agraph_t* g)
 {
     compute_bb(g);
-    spline_edges0(g, TRUE);
+    spline_edges0(g, true);
 }
 
 /* neato_layout:
@@ -1425,7 +1412,7 @@ void neato_layout(Agraph_t * g)
 	}
 	else gv_postprocess(g, 0);
     } else {
-	boolean noTranslate = mapBool(agget(g, "notranslate"), FALSE);
+	bool noTranslate = mapBool(agget(g, "notranslate"), false);
 	PSinputscale = get_inputscale (g);
 	neato_init_graph(g);
 	layoutMode = neatoMode(g);
@@ -1448,29 +1435,29 @@ void neato_layout(Agraph_t * g)
 	    graph_t **cc;
 	    int n_cc;
 	    int i;
-	    boolean pin;
+	    bool pin;
 
 	    cc = pccomps(g, &n_cc, cc_pfx, &pin);
 
 	    if (n_cc > 1) {
-		boolean *bp;
+		bool *bp;
 		for (i = 0; i < n_cc; i++) {
 		    gc = cc[i];
 		    nodeInduce(gc);
 		    neatoLayout(g, gc, layoutMode, model, &am);
 		    removeOverlapWith(gc, &am);
-		    setEdgeType (gc, ET_LINE);
+		    setEdgeType (gc, EDGETYPE_LINE);
 		    if (noTranslate) doEdges(gc);
 		    else spline_edges(gc);
 		}
 		if (pin) {
-		    bp = N_NEW(n_cc, boolean);
-		    bp[0] = TRUE;
+		    bp = N_NEW(n_cc, bool);
+		    bp[0] = true;
 		} else
-		    bp = 0;
-		pinfo.margin = Pack;
+		    bp = NULL;
+		pinfo.margin = (unsigned)Pack;
 		pinfo.fixed = bp;
-		pinfo.doSplines = 1;
+		pinfo.doSplines = true;
 		packGraphs(n_cc, cc, g, &pinfo);
 		free(bp);
 	    }
@@ -1505,3 +1492,13 @@ void neato_layout(Agraph_t * g)
     }
     PSinputscale = save_scale;
 }
+
+/**
+ * @dir lib/neatogen
+ * @brief "spring model" layout engine, API neatogen/neatoprocs.h
+ * @ingroup engines
+ *
+ * [Neato layout user manual](https://graphviz.org/docs/layouts/neato/)
+ *
+ * Other @ref engines
+ */
