@@ -11,6 +11,7 @@
 #pragma once
 
 #include <sfdpgen/post_process.h>
+#include <stdbool.h>
 
 typedef  StressMajorizationSmoother OverlapSmoother;
 
@@ -19,15 +20,15 @@ typedef  StressMajorizationSmoother OverlapSmoother;
 void OverlapSmoother_delete(OverlapSmoother sm);
 
 OverlapSmoother OverlapSmoother_new(SparseMatrix A, int m, 
-				    int dim, real lambda0, real *x, real *width, int include_original_graph, int neighborhood_only, 
-				    real *max_overlap, real *min_overlap,
+				    int dim, double lambda0, double *x, double *width, int include_original_graph, int neighborhood_only, 
+				    double *max_overlap, double *min_overlap,
 				    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, int shrink
 				    );
 
 enum {ELSCHEME_NONE = 0, ELSCHEME_PENALTY, ELSCHEME_PENALTY2, ELSCHEME_STRAIGHTLINE_PENALTY, ELSCHEME_STRAIGHTLINE_PENALTY2};
 
 struct relative_position_constraints_struct{
-  real constr_penalty; /* penalty parameter used in making edge labels as much on the line as possible */
+  double constr_penalty; /* penalty parameter used in making edge labels as much on the line as possible */
   int edge_labeling_scheme;/* specifying whether to treat node of the form |edgelabel|* as a special node representing an edge label. 
 			       0 (no action, default), 1 (penalty based method to make that kind of node close to the center of its neighbor), 
 			       2 (penalty based method to make that kind of node close to the "old" center of its neighbor), 
@@ -37,7 +38,7 @@ struct relative_position_constraints_struct{
   int *constr_nodes;/*constr_nodes: a list of nodes that need to be constrained. If NULL, unused.*/
   int *irn;/* working arrays to hold the Laplacian of the constrain graph */
   int *jcn;
-  real *val;
+  double *val;
   SparseMatrix A_constr; /*A_constr: neighbors of node i are in the row i of this matrix. i needs to sit
 			   in between these neighbors as much as possible. this must not be NULL
 			   if constr_nodes != NULL.*/
@@ -46,8 +47,9 @@ struct relative_position_constraints_struct{
 
 typedef struct relative_position_constraints_struct*  relative_position_constraints;
 
-real OverlapSmoother_smooth(OverlapSmoother sm, int dim, real *x);
+double OverlapSmoother_smooth(OverlapSmoother sm, int dim, double *x);
 
-void remove_overlap(int dim, SparseMatrix A, real *x, real *label_sizes, int ntry, real initial_scaling, 
-		    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, int doShrink);
-real overlap_scaling(int dim, int m, real *x, real *width, real scale_sta, real scale_sto, real epsilon, int maxiter);
+void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes,
+                    int ntry, double initial_scaling, int edge_labeling_scheme,
+                    int n_constr_nodes, int *constr_nodes,
+                    SparseMatrix A_constr, bool doShrink);
