@@ -16,11 +16,12 @@
 
 #include <math.h>
 #include <assert.h>
+#include <cgraph/alloc.h>
+#include <cgraph/startswith.h>
 #include <common/render.h>
 #include <pack/pack.h>
 #include <common/pointset.h>
-
-#define strneq(a,b,n)		(!strncmp(a,b,n))
+#include <stdbool.h>
 
 #define C 100			/* Max. avg. polyomino size */
 
@@ -97,7 +98,13 @@ static int cmpf(const void *X, const void *Y)
     const ginfo *x = *(ginfo *const *) X;
     const ginfo *y = *(ginfo *const *) Y;
     /* flip order to get descending array */
-    return y->perim - x->perim;
+    if (y->perim < x->perim) {
+        return -1;
+    }
+    if (y->perim > x->perim) {
+      return 1;
+    }
+    return 0;
 }
 
 /* fillLine:
@@ -159,8 +166,7 @@ static void fillLine(pointf p, pointf q, PointSet * ps)
  */
 static void
 fillEdge(Agedge_t * e, point p, PointSet * ps, int dx, int dy,
-	 int ssize, int doS)
-{
+         int ssize, bool doS) {
     int j, k;
     bezier bz;
     pointf pt, hpt;
@@ -286,7 +292,7 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
     int dx, dy;
     graph_t *subg;
     unsigned int margin = pinfo->margin;
-    int doSplines = pinfo->doSplines;
+    bool doSplines = pinfo->doSplines;
     box bb;
 
     if (root)
@@ -300,10 +306,9 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
 
     if (pinfo->mode == l_clust) {
 	int i;
-	void **alg;
 
 	/* backup the alg data */
-	alg = N_GNEW(agnnodes(g), void *);
+	void **alg = gv_calloc(agnnodes(g), sizeof(void*));
 	for (i = 0, n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    alg[i++] = ND_alg(n);
 	    ND_alg(n) = 0;
@@ -571,8 +576,8 @@ static int ucmpf(const void *X, const void *Y)
     int dX = userVals[x->index];
     int dY = userVals[y->index];
     if (dX > dY) return 1;
-    else if (dX < dY) return -1;
-    else return 0;
+    if (dX < dY) return -1;
+    return 0;
 }
 
 /* acmpf;
@@ -585,8 +590,8 @@ static int acmpf(const void *X, const void *Y)
     double dX = x->height + x->width; 
     double dY = y->height + y->width; 
     if (dX < dY) return 1;
-    else if (dX > dY) return -1;
-    else return 0;
+    if (dX > dY) return -1;
+    return 0;
 }
 
 #define INC(m,c,r) \
@@ -602,12 +607,8 @@ arrayRects (int ng, boxf* gs, pack_info* pinfo)
     int nr = 0, nc;
     int r, c;
     ainfo *info;
-    ainfo *ip;
-    ainfo **sinfo;
-    double* widths;
-    double* heights;
     double v, wd, ht;
-    point* places = N_NEW(ng, point);
+    point *places = gv_calloc(ng, sizeof(point));
     boxf bb;
     int sz, rowMajor;
 
@@ -637,10 +638,10 @@ arrayRects (int ng, boxf* gs, pack_info* pinfo)
     }
     if (Verbose)
 	fprintf (stderr, "array packing: %s %d rows %d columns\n", (rowMajor?"row major":"column major"), nr, nc);
-    widths = N_NEW(nc+1, double);
-    heights = N_NEW(nr+1, double);
+    double *widths = gv_calloc(nc + 1, sizeof(double));
+    double *heights = gv_calloc(nr + 1, sizeof(double));
 
-    ip = info = N_NEW(ng, ainfo);
+    ainfo *ip = info = gv_calloc(ng, sizeof(ainfo));
     for (i = 0; i < ng; i++, ip++) {
 	bb = gs[i];
 	ip->width = bb.UR.x - bb.LL.x + pinfo->margin;
@@ -648,7 +649,7 @@ arrayRects (int ng, boxf* gs, pack_info* pinfo)
 	ip->index = i;
     }
 
-    sinfo = N_NEW(ng, ainfo*);
+    ainfo **sinfo = gv_calloc(ng, sizeof(ainfo*));
     for (i = 0; i < ng; i++) {
 	sinfo[i] = info + i;
     }
@@ -719,9 +720,6 @@ static point*
 polyRects(int ng, boxf* gs, pack_info * pinfo)
 {
     int stepSize;
-    ginfo *info;
-    ginfo **sinfo;
-    point *places;
     Dict_t *ps;
     int i;
     point center;
@@ -735,21 +733,21 @@ polyRects(int ng, boxf* gs, pack_info * pinfo)
 
     /* generate polyomino cover for the rectangles */
     center.x = center.y = 0;
-    info = N_NEW(ng, ginfo);
+    ginfo *info = gv_calloc(ng, sizeof(ginfo));
     for (i = 0; i < ng; i++) {
 	info[i].index = i;
 	genBox(gs[i], info + i, stepSize, pinfo->margin, center, "");
     }
 
     /* sort */
-    sinfo = N_NEW(ng, ginfo *);
+    ginfo **sinfo = gv_calloc(ng, sizeof(ginfo*));
     for (i = 0; i < ng; i++) {
 	sinfo[i] = info + i;
     }
     qsort(sinfo, ng, sizeof(ginfo *), cmpf);
 
     ps = newPS();
-    places = N_NEW(ng, point);
+    point *places = gv_calloc(ng, sizeof(point));
     for (i = 0; i < ng; i++)
 	placeGraph(i, sinfo[i], ps, places + sinfo[i]->index,
 		       stepSize, pinfo->margin, gs);
@@ -798,15 +796,12 @@ polyGraphs(int ng, Agraph_t ** gs, Agraph_t * root, pack_info * pinfo)
 {
     int stepSize;
     ginfo *info;
-    ginfo **sinfo;
-    point *places;
     Dict_t *ps;
     int i;
     bool *fixed = pinfo->fixed;
     int fixed_cnt = 0;
     box bb, fixed_bb = { {0, 0}, {0, 0} };
     point center;
-    boxf* bbs;
 
     if (ng <= 0)
 	return 0;
@@ -835,7 +830,7 @@ polyGraphs(int ng, Agraph_t ** gs, Agraph_t * root, pack_info * pinfo)
     }
 
     /* calculate grid size */
-    bbs = N_GNEW(ng, boxf);
+    boxf *bbs = gv_calloc(ng, sizeof(boxf));
     for (i = 0; i < ng; i++)
 	bbs[i] = GD_bb(gs[i]);
     stepSize = computeStep(ng, bbs, pinfo->margin);
@@ -850,7 +845,7 @@ polyGraphs(int ng, Agraph_t ** gs, Agraph_t * root, pack_info * pinfo)
 	center.y = (fixed_bb.LL.y + fixed_bb.UR.y) / 2;
     } else
 	center.x = center.y = 0;
-    info = N_NEW(ng, ginfo);
+    info = gv_calloc(ng, sizeof(ginfo));
     for (i = 0; i < ng; i++) {
 	Agraph_t *g = gs[i];
 	info[i].index = i;
@@ -862,14 +857,14 @@ polyGraphs(int ng, Agraph_t ** gs, Agraph_t * root, pack_info * pinfo)
     }
 
     /* sort */
-    sinfo = N_NEW(ng, ginfo *);
+    ginfo **sinfo = gv_calloc(ng, sizeof(ginfo*));
     for (i = 0; i < ng; i++) {
 	sinfo[i] = info + i;
     }
     qsort(sinfo, ng, sizeof(ginfo *), cmpf);
 
     ps = newPS();
-    places = N_NEW(ng, point);
+    point *places = gv_calloc(ng, sizeof(point));
     if (fixed) {
 	for (i = 0; i < ng; i++) {
 	    if (fixed[i])
@@ -905,7 +900,6 @@ point *putGraphs(int ng, Agraph_t ** gs, Agraph_t * root,
 		 pack_info * pinfo)
 {
     int i, v;
-    boxf* bbs;
     Agraph_t* g;
     point* pts = NULL;
     char* s;
@@ -915,7 +909,7 @@ point *putGraphs(int ng, Agraph_t ** gs, Agraph_t * root,
     if (pinfo->mode <= l_graph)
 	return polyGraphs (ng, gs, root, pinfo);
     
-    bbs = N_GNEW(ng, boxf);
+    boxf *bbs = gv_calloc(ng, sizeof(boxf));
 
     for (i = 0; i < ng; i++) {
 	g = gs[i];
@@ -925,7 +919,7 @@ point *putGraphs(int ng, Agraph_t ** gs, Agraph_t * root,
 
     if (pinfo->mode == l_array) {
 	if (pinfo->flags & PK_USER_VALS) {
-	    pinfo->vals = N_NEW(ng, packval_t);
+	    pinfo->vals = gv_calloc(ng, sizeof(packval_t));
 	    for (i = 0; i < ng; i++) {
 		s = agget (gs[i], "sortv");
 		if (s && sscanf(s, "%d", &v) > 0 && v >= 0)
@@ -950,10 +944,9 @@ putRects(int ng, boxf* bbs, pack_info* pinfo)
     if (pinfo->mode == l_node || pinfo->mode == l_clust) return NULL;
     if (pinfo->mode == l_graph)
 	return polyRects (ng, bbs, pinfo);
-    else if (pinfo->mode == l_array)
+    if (pinfo->mode == l_array)
 	return arrayRects (ng, bbs, pinfo);
-    else
-	return NULL;
+    return NULL;
 }
 
 /* packRects:
@@ -1073,8 +1066,7 @@ static void shiftGraph(Agraph_t * g, int dx, int dy)
  * the spl field in Aedgeinfo_t.
  */
 int
-shiftGraphs(int ng, Agraph_t ** gs, point * pp, Agraph_t * root,
-	    int doSplines)
+shiftGraphs(int ng, Agraph_t ** gs, point * pp, Agraph_t * root, bool doSplines)
 {
     int i;
     int dx, dy;
@@ -1181,17 +1173,13 @@ pack_graph(int ng, Agraph_t** gs, Agraph_t* root, bool *fixed)
     pack_info info;
 
     getPackInfo(root, l_graph, CL_OFFSET, &info);
-    info.doSplines = 1;
+    info.doSplines = true;
     info.fixed = fixed;
     ret = packSubgraphs(ng, gs, root, &info);
     if (ret == 0) dotneato_postprocess (root);
     return ret;
 }
 
-#define ARRAY  "array"
-#define ASPECT "aspect"
-#define SLEN(s) (sizeof(s)/sizeof(char) - 1)
-
 static const char*chkFlags(const char *p, pack_info *pinfo) {
     int c, more;
 
@@ -1236,33 +1224,24 @@ static const char*chkFlags(const char *p, pack_info *pinfo) {
     return p;
 }
 
-static char*
-mode2Str (pack_mode m)
-{
-    char *s;
+static const char *mode2Str(pack_mode m) {
 
     switch (m) {
     case l_clust:
-	s = "cluster";
-	break;
+	return "cluster";
     case l_node:
-	s = "node";
-	break;
+	return "node";
     case l_graph:
-	s = "graph";
-	break;
+	return "graph";
     case l_array:
-	s = "array";
-	break;
+	return "array";
     case l_aspect:
-	s = "aspect";
-	break;
+	return "aspect";
     case l_undef: 
     default:
-	s = "undefined";
 	break;
     }
-    return s;
+    return "undefined";
 }
 
 /* parsePackModeInfo;
@@ -1283,16 +1262,16 @@ parsePackModeInfo(const char* p, pack_mode dflt, pack_info* pinfo)
     if (p && *p) {
 	switch (*p) {
 	case 'a':
-	    if (strneq(p, ARRAY, SLEN(ARRAY))) {
+	    if (startswith(p, "array")) {
 		pinfo->mode = l_array;
-		p += SLEN(ARRAY);
+		p += strlen("array");
 		p = chkFlags (p, pinfo);
 		if ((sscanf (p, "%d", &i)>0) && (i > 0))
 		    pinfo->sz = i;
 	    }
-	    else if (strneq(p, ASPECT, SLEN(ASPECT))) {
+	    else if (startswith(p, "aspect")) {
 		pinfo->mode = l_aspect;
-		if (sscanf(p + SLEN(ARRAY), "%f", &v) > 0 && v > 0)
+		if (sscanf(p + strlen("aspect"), "%f", &v) > 0 && v > 0)
 		    pinfo->aspect = v;
 		else
 		    pinfo->aspect = 1;
@@ -1371,9 +1350,16 @@ getPackInfo(Agraph_t * g, pack_mode dflt, int dfltMargin, pack_info* pinfo)
     if (Verbose) {
 	fprintf (stderr, "  margin %u\n", pinfo->margin);
     }
-    pinfo->doSplines = 0;
+    pinfo->doSplines = false;
     pinfo->fixed = NULL;
     getPackModeInfo(g, dflt, pinfo);
 
     return pinfo->mode;
 }
+/**
+ * @dir lib/pack
+ * @brief support for connected components, API pack.h
+ *
+ * [man 3 libpack](https://graphviz.org/pdf/pack.3.pdf)
+ *
+ */
