@@ -1,3 +1,24 @@
+/**
+ * @file
+ * @brief support for connected components
+ * @ingroup public_apis
+ *
+ * **libpack** supports the use of connected components
+ * in the context of laying out graphs using other graphviz libraries.
+ * One set of functions can be used to take a single graph and break it
+ * apart into connected components.
+ * A complementary set of functions takes a collection of graphs
+ * (not necessarily components of a single graph) which have been laid
+ * out separately, and packs them together.
+ *
+ * As this library is meant to be used with `libcommon`, it relies on the
+ * @ref Agraphinfo_t, @ref Agnodeinfo_t and @ref Agedgeinfo_t used in
+ * that library.
+ *
+ * [man 3 libpack](https://graphviz.org/pdf/pack.3.pdf)
+ *
+ */
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,10 +29,9 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
 
-
-#ifndef _PACK_H
-#define _PACK_H 1
+#include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -47,22 +67,23 @@ typedef unsigned int packval_t;
 	float aspect;		/* desired aspect ratio */
 	int sz;			/* row/column size size */
 	unsigned int margin;	/* margin left around objects, in points */
-	int doSplines;		/* use splines in constructing graph shape */
+	bool doSplines; ///< use splines in constructing graph shape
 	pack_mode mode;		/* granularity and method */
-	boolean *fixed;		/* fixed[i] == true implies g[i] should not be moved */
+	bool *fixed;		/* fixed[i] == true implies g[i] should not be moved */
 	packval_t* vals;	/* for arrays, sort numbers */
 	int flags;       
     } pack_info;
 
-/*visual studio*/
-#ifdef _WIN32
-#ifndef GVC_EXPORTS
+#ifdef GVDLL
+#ifdef GVC_EXPORTS
+#define PACK_API __declspec(dllexport)
+#else
 #define PACK_API __declspec(dllimport)
 #endif
 #endif
-/*end visual studio*/
+
 #ifndef PACK_API
-#define PACK_API extern
+#define PACK_API /* nothing */
 #endif
 
     PACK_API point *putRects(int ng, boxf* bbs, pack_info* pinfo);
@@ -71,24 +92,24 @@ typedef unsigned int packval_t;
     PACK_API point *putGraphs(int, Agraph_t **, Agraph_t *, pack_info *);
     PACK_API int packGraphs(int, Agraph_t **, Agraph_t *, pack_info *);
     PACK_API int packSubgraphs(int, Agraph_t **, Agraph_t *, pack_info *);
-    PACK_API int pack_graph(int ng, Agraph_t** gs, Agraph_t* root, boolean* fixed);
+    PACK_API int pack_graph(int ng, Agraph_t** gs, Agraph_t* root, bool *fixed);
 
-    PACK_API int shiftGraphs(int, Agraph_t**, point*, Agraph_t*, int);
+    PACK_API int shiftGraphs(int, Agraph_t**, point*, Agraph_t*, bool);
 
     PACK_API pack_mode getPackMode(Agraph_t * g, pack_mode dflt);
     PACK_API int getPack(Agraph_t *, int not_def, int dflt);
     PACK_API pack_mode getPackInfo(Agraph_t * g, pack_mode dflt, int dfltMargin, pack_info*);
     PACK_API pack_mode getPackModeInfo(Agraph_t * g, pack_mode dflt, pack_info*);
-    PACK_API pack_mode parsePackModeInfo(char* p, pack_mode dflt, pack_info* pinfo);
+    PACK_API pack_mode parsePackModeInfo(const char* p, pack_mode dflt,
+                                         pack_info* pinfo);
 
     PACK_API int isConnected(Agraph_t *);
     PACK_API Agraph_t **ccomps(Agraph_t *, int *, char *);
     PACK_API Agraph_t **cccomps(Agraph_t *, int *, char *);
-    PACK_API Agraph_t **pccomps(Agraph_t *, int *, char *, boolean *);
+    PACK_API Agraph_t **pccomps(Agraph_t *, int *, char *, bool *);
     PACK_API int nodeInduce(Agraph_t *);
     PACK_API Agraph_t *mapClust(Agraph_t *);
 #undef PACK_API
 #ifdef __cplusplus
 }
 #endif
-#endif
