@@ -9,7 +9,8 @@
  *************************************************************************/
 
 #define STANDALONE
-#include <cgraph/itos.h>
+#include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
 #include <sparse/general.h>
 #include <sparse/DotIO.h>
 #include <sparse/clustering.h>
@@ -27,20 +28,19 @@ typedef struct {
 
 #define ND_id(n)  (((Agnodeinfo_t*)((n)->base.data))->id)
 
-static void color_string(int slen, char *buf, int dim, double *color){
+static void color_string(agxbuf *buf, int dim, double *color){
   if (dim > 3 || dim < 1){
     fprintf(stderr,"can only 1, 2 or 3 dimensional color space. with color value between 0 to 1\n");
     assert(0);
   }
-  assert(slen >= 3);
   if (dim == 3){
-    sprintf(buf,"#%02x%02x%02x", MIN((unsigned int)(color[0]*255),255), 
+    agxbprint(buf, "#%02x%02x%02x", MIN((unsigned int)(color[0] *255), 255),
 	    MIN((unsigned int) (color[1]*255), 255), MIN((unsigned int)(color[2]*255), 255));
   } else if (dim == 1){
-    sprintf(buf,"#%02x%02x%02x", MIN((unsigned int)(color[0]*255),255), 
+    agxbprint(buf, "#%02x%02x%02x", MIN((unsigned int)(color[0] * 255), 255),
 	    MIN((unsigned int) (color[0]*255), 255), MIN((unsigned int)(color[0]*255), 255));
   } else if (dim == 2){
-    sprintf(buf,"#%02x%02x%02x", MIN((unsigned int)(color[0]*255),255), 
+    agxbprint(buf, "#%02x%02x%02x", MIN((unsigned int)(color[0] * 255), 255),
 	    0, MIN((unsigned int)(color[1]*255), 255));
   }
 }
@@ -51,8 +51,7 @@ void attach_edge_colors(Agraph_t* g, int dim, double *colors){
   Agsym_t* sym = agattr(g, AGEDGE, "color", 0); 
   Agedge_t* e;
   Agnode_t* n;
-  enum {slen = 1024};
-  char buf[slen];
+  agxbuf buf = {0};
   unsigned row, col;
   int ie = 0;
 
@@ -64,38 +63,33 @@ void attach_edge_colors(Agraph_t* g, int dim, double *colors){
     for (e = agfstout (g, n); e; e = agnxtout (g, e)) {
       col = ND_id(aghead(e));
       if (row == col) continue;
-      color_string(slen, buf, dim, colors + ie*dim);
-      agxset(e, sym, buf);
+      color_string(&buf, dim, colors + ie*dim);
+      agxset(e, sym, agxbuse(&buf));
       ie++;
     }
   }
-
+  agxbfree(&buf);
 }
 
 /* SparseMatrix_import_dot:
  * Assumes g is connected and simple, i.e., we can have a->b and b->a
  * but not a->b and a->b
  */
-SparseMatrix 
-SparseMatrix_import_dot (Agraph_t* g, int dim, double **label_sizes, double **x, int *n_edge_label_nodes, int **edge_label_nodes, int format, SparseMatrix *D)
-{
+SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
+                                     double **x, int format) {
   SparseMatrix A = 0;
   Agnode_t* n;
   Agedge_t* e;
-  Agsym_t *sym, *symD = NULL;
+  Agsym_t *sym;
   Agsym_t *psym;
   int nnodes;
   int nedges;
   int i, row;
   int* I;
   int* J;
-  double *val, *valD = NULL;
+  double *val;
   double v;
   int type = MATRIX_TYPE_REAL;
-  double padding = 10;
-  int nedge_nodes = 0;
-
-
 
   if (!g) return NULL;
   nnodes = agnnodes (g);
@@ -115,21 +109,16 @@ SparseMatrix_import_dot (Agraph_t* g, int dim, double **label_sizes, double **x,
     A->nz = nedges;
     I = A->ia;
     J = A->ja;
-    val = (double*) A->a;
+    val = A->a;
   } else {
-    I = N_NEW(nedges, int);
-    J = N_NEW(nedges, int);
-    val = N_NEW(nedges, double);
+    I = gv_calloc(nedges, sizeof(int));
+    J = gv_calloc(nedges, sizeof(int));
+    val = gv_calloc(nedges, sizeof(double));
   }
 
   sym = agattr(g, AGEDGE, "weight", NULL);
-  if (D) {
-    symD = agattr(g, AGEDGE, "len", NULL);
-    valD = N_NEW(nedges, double);
-  }
   i = 0;
   for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
-    if (edge_label_nodes && strncmp(agnameof(n), "|edgelabel|",11)==0) nedge_nodes++;
     row = ND_id(n);
     for (e = agfstout (g, n); e; e = agnxtout (g, e)) {
       I[i] = row;
@@ -143,54 +132,15 @@ SparseMatrix_import_dot (Agraph_t* g, int dim, double **label_sizes, double **x,
       }
       val[i] = v;
 
-      /* edge length */
-      if (symD) {
-        if (sscanf (agxget (e, symD), "%lf", &v) != 1) {
-          v = 72;
-        } else {
-          v *= 72;/* len is specified in inch. Convert to points */
-        }
-	valD[i] = v;
-      } else if (valD) {
-        valD[i] = 72;
-      }
-
       i++;
     }
   }
   
-  if (edge_label_nodes) {
-    *edge_label_nodes = MALLOC(sizeof(int)*nedge_nodes);
-    nedge_nodes = 0;
-  }
-
-
-  if (label_sizes) *label_sizes = MALLOC(sizeof(double)*2*nnodes);
-  for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
-    double sz;
-    i = ND_id(n);
-    if (edge_label_nodes && strncmp(agnameof(n), "|edgelabel|",11)==0) {
-      (*edge_label_nodes)[nedge_nodes++] = i;
-    }
-    if (label_sizes){
-      if (agget(n, "width") && agget(n, "height")){
-	sscanf(agget(n, "width"), "%lf", &sz);
-	(*label_sizes)[i*2] = POINTS(sz)*.5 + padding*0.5;
-	sscanf(agget(n, "height"), "%lf", &sz);
-	(*label_sizes)[i*2+1] = POINTS(sz)*.5  + padding*0.5;
-      } else {
-	(*label_sizes)[i*2] = 4*POINTS(0.75)*.5;
-	(*label_sizes)[i*2+1] = 4*POINTS(0.5)*.5;
-      }
-    }
-  }
-
   if (x && (psym = agattr(g, AGNODE, "pos", NULL))) {
     bool has_positions = true;
     char* pval;
     if (!(*x)) {
-      *x = MALLOC(sizeof(double)*dim*nnodes);
-      assert(*x);
+      *x = gv_calloc(dim * nnodes, sizeof(double));
     }
     for (n = agfstnode (g); n && has_positions; n = agnxtnode (g, n)) {
       double xx,yy, zz,ww;
@@ -249,9 +199,6 @@ SparseMatrix_import_dot (Agraph_t* g, int dim, double **label_sizes, double **x,
 
   size_t sz = sizeof(double);
   if (format == FORMAT_CSR) A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, val, type, sz);
-  if (edge_label_nodes) *n_edge_label_nodes = nedge_nodes;
-
-  if (D) *D = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, valD, type, sz);
 
 done:
   if (format != FORMAT_COORD){
@@ -259,7 +206,6 @@ done:
     free(J);
     free(val);
   }
-  free(valD);
 
   return A;
 }
@@ -314,15 +260,16 @@ static int hex2int(char h){
   if (h >= 'a' && h <= 'f') return 10 + h - 'a';
   if (h >= 'A' && h <= 'F') return 10 + h - 'A';
   return 0;
-}  
-static float hexcol2rgb(char *h){
+}
+
+static float hexcol2rgb(const char *h) {
   return (hex2int(h[0])*16 + hex2int(h[1]))/255.;
 }
 
 void Dot_SetClusterColor(Agraph_t* g, float *rgb_r,  float *rgb_g,  float *rgb_b, int *clusters){
 
   Agnode_t* n;
-  char scluster[20];
+  agxbuf scluster = {0};
   unsigned i;
   Agsym_t* clust_clr_sym = agattr(g, AGNODE, "clustercolor", NULL); 
 
@@ -330,10 +277,12 @@ void Dot_SetClusterColor(Agraph_t* g, float *rgb_r,  float *rgb_g,  float *rgb_b
   for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
     i = ND_id(n);
     if (rgb_r && rgb_g && rgb_b) {
-      rgb2hex((rgb_r)[(clusters)[i]],(rgb_g)[(clusters)[i]],(rgb_b)[(clusters)[i]], scluster, NULL);
+      rgb2hex(rgb_r[clusters[i]], rgb_g[clusters[i]], rgb_b[clusters[i]],
+              &scluster, NULL);
     }
-    agxset(n,clust_clr_sym,scluster);
+    agxset(n, clust_clr_sym, agxbuse(&scluster));
   }
+  agxbfree(&scluster);
 }
 
 SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim, int *nn, double **label_sizes, double **x, int **clusters, float **rgb_r,  float **rgb_g,  float **rgb_b,  float **fsz, char ***labels, int default_color_scheme, int clustering_scheme, int useClusters){
@@ -346,9 +295,6 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
   int nnodes;
   int nedges;
   int i, row, ic,nc, j;
-  int* I;
-  int* J;
-  double* val;
   double v;
   int type = MATRIX_TYPE_REAL;
   char scluster[100];
@@ -413,9 +359,9 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
     ND_id(n) = i++;
 
   /* form matrix */  
-  I = N_NEW(nedges, int);
-  J = N_NEW(nedges, int);
-  val = N_NEW(nedges, double);
+  int* I = gv_calloc(nedges, sizeof(int));
+  int* J = gv_calloc(nedges, sizeof(int));
+  double* val = gv_calloc(nedges, sizeof(double));
 
   sym = agattr(g, AGEDGE, "weight", NULL); 
   clust_sym = agattr(g, AGNODE, "cluster", NULL); 
@@ -440,7 +386,7 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
                                           type, sizeof(double));
 
   /* get clustering info */
-  *clusters = MALLOC(sizeof(int)*nnodes);
+  *clusters = gv_calloc(nnodes, sizeof(int));
   nc = 1;
   MIN_GRPS = 0;
   /* if useClusters, the nodes in each top-level cluster subgraph are assigned to
@@ -492,16 +438,15 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
   MAX_GRPS = nc;
 
   if (noclusterinfo) {
-    int use_value = TRUE, flag = 0;
     double modularity;
     if (!clust_sym) clust_sym = agattr(g,AGNODE,"cluster","-1");
 
     if (clustering_scheme == CLUSTERING_MQ){
-      mq_clustering(A, FALSE, maxcluster, use_value,
-		    &nc, clusters, &modularity, &flag);
+      mq_clustering(A, maxcluster,
+		    &nc, clusters, &modularity);
     } else if (clustering_scheme == CLUSTERING_MODULARITY){ 
-      modularity_clustering(A, FALSE, maxcluster, use_value,
-		    &nc, clusters, &modularity, &flag);
+      modularity_clustering(A, false, maxcluster,
+		    &nc, clusters, &modularity);
     } else {
       assert(0);
     }
@@ -518,18 +463,18 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
     }
   }
 
-  *label_sizes = MALLOC(sizeof(double)*dim*nnodes);
+  *label_sizes = gv_calloc(dim * nnodes, sizeof(double));
   if (pal || (!noclusterinfo && clust_clr_sym)){
-    *rgb_r = MALLOC(sizeof(float)*(1+MAX_GRPS));
-    *rgb_g = MALLOC(sizeof(float)*(1+MAX_GRPS));
-    *rgb_b = MALLOC(sizeof(float)*(1+MAX_GRPS));
+    *rgb_r = gv_calloc(1 + MAX_GRPS, sizeof(float));
+    *rgb_g = gv_calloc(1 + MAX_GRPS, sizeof(float));
+    *rgb_b = gv_calloc(1 + MAX_GRPS, sizeof(float));
   } else {
     *rgb_r = NULL;
     *rgb_g = NULL;
     *rgb_b = NULL;
   }
-  *fsz = MALLOC(sizeof(float)*nnodes);
-  *labels = MALLOC(sizeof(char*)*nnodes);
+  *fsz = gv_calloc(nnodes, sizeof(float));
+  *labels = gv_calloc(nnodes, sizeof(char*));
 
   for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
     gvcolor_t color;
@@ -580,9 +525,8 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
      (*rgb_b)[(*clusters)[i]] = color.u.RGBA[2];
    }
 
-   if (!noclusterinfo && agget(n, "cluster") && agget(n, "clustercolor") && strlen(agget(n, "clustercolor") ) >= 7 && pal){
-     char cc[10];
-     strcpy(cc, agget(n, "clustercolor"));
+   const char *cc = agget(n, "clustercolor");
+   if (!noclusterinfo && agget(n, "cluster") && cc && strlen(cc) >= 7 && pal) {
      (*rgb_r)[(*clusters)[i]] = hexcol2rgb(cc+1);
      (*rgb_g)[(*clusters)[i]] = hexcol2rgb(cc+3);
      (*rgb_b)[(*clusters)[i]] = hexcol2rgb(cc+5);
@@ -593,7 +537,7 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
 
   if (x){
     bool has_position = false;
-    *x = MALLOC(sizeof(double)*dim*nnodes);
+    *x = gv_calloc(dim * nnodes, sizeof(double));
     for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
       double xx,yy;
       i = ND_id(n);
@@ -630,17 +574,10 @@ void attached_clustering(Agraph_t* g, int maxcluster, int clustering_scheme){
   int nnodes;
   int nedges;
   int i, row,nc;
-  int* I;
-  int* J;
-  double* val;
   double v;
   int type = MATRIX_TYPE_REAL;
   size_t sz = sizeof(double);
 
-  int *clusters;
-
-
-  
   if (!g) return;
   nnodes = agnnodes (g);
   nedges = agnedges (g);
@@ -651,9 +588,9 @@ void attached_clustering(Agraph_t* g, int maxcluster, int clustering_scheme){
     ND_id(n) = i++;
   
   /* form matrix */  
-  I = N_NEW(nedges, int);
-  J = N_NEW(nedges, int);
-  val = N_NEW(nedges, double);
+  int* I = gv_calloc(nedges, sizeof(int));
+  int* J = gv_calloc(nedges, sizeof(int));
+  double* val = gv_calloc(nedges, sizeof(double));
   
   sym = agattr(g, AGEDGE, "weight", NULL);
   clust_sym = agattr(g, AGNODE, "cluster", NULL);
@@ -676,26 +613,28 @@ void attached_clustering(Agraph_t* g, int maxcluster, int clustering_scheme){
   }
   A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, val, type, sz);
 
-  clusters = MALLOC(sizeof(int)*nnodes);
+  int *clusters = gv_calloc(nnodes, sizeof(int));
 
   {
-    int use_value = TRUE, flag = 0;
     double modularity;
     if (!clust_sym) clust_sym = agattr(g,AGNODE,"cluster","-1");
     
     if (clustering_scheme == CLUSTERING_MQ){
-      mq_clustering(A, FALSE, maxcluster, use_value,
-		    &nc, &clusters, &modularity, &flag);
+      mq_clustering(A, maxcluster,
+		    &nc, &clusters, &modularity);
     } else if (clustering_scheme == CLUSTERING_MODULARITY){ 
-      modularity_clustering(A, FALSE, maxcluster, use_value,
-			    &nc, &clusters, &modularity, &flag);
+      modularity_clustering(A, false, maxcluster,
+			    &nc, &clusters, &modularity);
     } else {
       assert(0);
     }
     for (i = 0; i < nnodes; i++) (clusters)[i]++;/* make into 1 based */
     for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
       i = ND_id(n);
-      agxset(n, clust_sym, itos((clusters)[i]).str);
+      agxbuf value_buffer = {0};
+      agxbprint(&value_buffer, "%d", clusters[i]);
+      agxset(n, clust_sym, agxbuse(&value_buffer));
+      agxbfree(&value_buffer);
     }
     if (Verbose){
       fprintf(stderr," no complement clustering info in dot file, using modularity clustering. Modularity = %f, ncluster=%d\n",modularity, nc);
@@ -715,7 +654,7 @@ void attached_clustering(Agraph_t* g, int maxcluster, int clustering_scheme){
 
 void initDotIO (Agraph_t *g)
 {
-  aginit(g, AGNODE, "info", sizeof(Agnodeinfo_t), TRUE);
+  aginit(g, AGNODE, "info", sizeof(Agnodeinfo_t), true);
 }
 
 void setDotNodeID (Agnode_t* n, int v)
