@@ -14,6 +14,7 @@
 #include <io.h>
 #endif
 
+#include <limits.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -33,8 +34,6 @@
 #include <gvc/gvc.h>
 #include <gvc/gvio.h>
 
-/* #define NEW_XDOT */
-
 typedef enum {
 	FORMAT_DOT,
 	FORMAT_CANON,
@@ -93,63 +92,25 @@ static void xdot_str (GVJ_t *job, char* pfx, const char* s)
     xdot_str_xbuf (xbufs[emit_state], pfx, s);
 }
 
-/* xdot_trim_zeros
- * Trailing zeros are removed and decimal point, if possible.
- * Add trailing space if addSpace is non-zero.
- */
-static void xdot_trim_zeros (char* buf, int addSpace)
-{
-    char* dotp;
-    char* p;
-
-    if ((dotp = strchr (buf,'.'))) {
-	p = dotp+1;
-	while (*p) p++;  // find end of string
-	p--;
-	while (*p == '0') *p-- = '\0';
-        if (*p == '.')        // If all decimals were zeros, remove ".".
-            *p = '\0';
-	else
-	    p++;
-    }
-    else if (addSpace)
-	p = buf + strlen(buf);
-
-    if (addSpace) { /* p points to null byte */
-	*p++ = ' ';
-	*p = '\0';
-    }
-}
-
 /* xdot_fmt_num:
  * Convert double to string with space at end.
  * Trailing zeros are removed and decimal point, if possible.
  */
-static void xdot_fmt_num(char *buf, size_t buf_size, double v) {
-    // Prevents values like -0
-    if (v > -0.00000001 && v < 0.00000001)
-    {
-        snprintf(buf, buf_size, "0 ");
-        return;
-    }
-    snprintf(buf, buf_size, "%.02f", v);
-    xdot_trim_zeros (buf, 1);
+static void xdot_fmt_num(agxbuf *buf, double v) {
+  agxbprint(buf, "%.02f", v);
+  agxbuf_trim_zeros(buf);
+  agxbputc(buf, ' ');
 }
 
 static void xdot_point(agxbuf *xb, pointf p)
 {
-    char buf[BUFSIZ];
-    xdot_fmt_num(buf, sizeof(buf), p.x);
-    agxbput(xb, buf);
-    xdot_fmt_num(buf, sizeof(buf), yDir(p.y));
-    agxbput(xb, buf);
+  xdot_fmt_num(xb, p.x);
+  xdot_fmt_num(xb, yDir(p.y));
 }
 
 static void xdot_num(agxbuf *xb, double v)
 {
-    char buf[BUFSIZ];
-    xdot_fmt_num(buf, sizeof(buf), v);
-    agxbput(xb, buf);
+  xdot_fmt_num(xb, v);
 }
 
 static void xdot_points(GVJ_t *job, char c, pointf * A, int n)
@@ -187,28 +148,25 @@ static void xdot_fillcolor (GVJ_t *job)
 
 static void xdot_style (GVJ_t *job)
 {
-    char buf0[BUFSIZ];
-    char buf [128]; /* enough to hold a double */
-    agxbuf xb;
+    agxbuf xb = {0};
     char* p, **s;
-    int more;
-
-    agxbinit(&xb, BUFSIZ, buf0);
 
     /* First, check if penwidth state is correct */
     if (job->obj->penwidth != penwidth[job->obj->emit_state]) {
 	penwidth[job->obj->emit_state] = job->obj->penwidth;
 	agxbput (&xb, "setlinewidth(");
-	snprintf(buf, sizeof(buf), "%.3f", job->obj->penwidth);
-	xdot_trim_zeros (buf, 0);
-	agxbprint(&xb, "%s)", buf);
+	agxbprint(&xb, "%.3f", job->obj->penwidth);
+	agxbuf_trim_zeros(&xb);
+	agxbputc(&xb, ')');
         xdot_str (job, "S ", agxbuse(&xb));
     }
 
     /* now process raw style, if any */
     s = job->obj->rawstyle;
-    if (!s)
+    if (!s) {
+	agxbfree(&xb);
 	return;
+    }
 
     while ((p = *s++)) {
 	if (streq(p, "filled") || streq(p, "bold") || streq(p, "setlinewidth")) continue;
@@ -218,14 +176,10 @@ static void xdot_style (GVJ_t *job)
         p++;
         if (*p) {  /* arguments */
             agxbputc(&xb, '(');
-            more = 0;
-            while (*p) {
-                if (more)
-                    agxbputc(&xb, ',');
-                agxbput(&xb, p);
+            for (const char *separator = ""; *p; separator = ",") {
+                agxbprint(&xb, "%s%s", separator, p);
                 while (*p) p++;
                 p++;
-                more++;
             }
             agxbputc(&xb, ')');
         }
@@ -311,40 +265,6 @@ static void xdot_end_edge(GVJ_t* job)
     textflags[EMIT_HLABEL] = 0;
 }
 
-#ifdef NEW_XDOT
-/* xdot_begin_anchor:
- * The encoding of which fields are present assumes that one of the fields is present,
- * so there is never a 0 after the H.
- */
-static void xdot_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target, char *id)
-{
-    emit_state_t emit_state = job->obj->emit_state;
-    unsigned int flags = 0;
-
-    agxbput(xbufs[emit_state], "H ");
-    if (href)
-	flags |= 1;
-    if (tooltip)
-	flags |= 2;
-    if (target)
-	flags |= 4;
-    agxbprint(xbufs[emit_state], "%d ", flags);
-    if (href)
-	xdot_str (job, "", href);
-    if (tooltip)
-	xdot_str (job, "", tooltip);
-    if (target)
-	xdot_str (job, "", target);
-}
-
-static void xdot_end_anchor(GVJ_t * job)
-{
-    emit_state_t emit_state = job->obj->emit_state;
-
-    agxbput(xbufs[emit_state], "H 0 ");
-}
-#endif
-
 static void xdot_end_cluster(GVJ_t * job)
 {
     Agraph_t* cluster_g = job->obj->u.sg;
@@ -358,27 +278,20 @@ static void xdot_end_cluster(GVJ_t * job)
     textflags[EMIT_CLABEL] = 0;
 }
 
-static unsigned short
-versionStr2Version (char* str)
-{
-    char c, buf[BUFSIZ];
-    int n = 0;
-    char* s = str;
-    unsigned short us;
-
-    while ((c = *s++)) {
-	if (isdigit(c)) {
-	    if (n < BUFSIZ-1) buf[n++] = c;
-	    else {
-		agerr(AGWARN, "xdot version \"%s\" too long", str);
-		break;
-	    }
-	}
+static unsigned short versionStr2Version(const char *str) {
+  unsigned short us = 0;
+  for (size_t i = 0; str[i] != '\0'; ++i) {
+    if (!isdigit((int)str[i])) {
+      continue;
     }
-    buf[n] = '\0';
-    
-    us = atoi(buf);
-    return us;
+    unsigned short digit = (unsigned short)(str[i] - '0');
+    if (us > (USHRT_MAX - digit) / 10) {
+      agerr(AGWARN, "xdot version \"%s\" too long", str);
+      break;
+    }
+    us = (unsigned short)(us * 10 + digit);
+  }
+  return us;
 }
 
 /* 
@@ -558,18 +471,16 @@ static void dot_end_graph(GVJ_t *job)
     g->clos->disc.io = io_save;
 }
 
-static unsigned int flag_masks[] = { 0x1F, 0x3F, 0x7F };
+static const unsigned int flag_masks[] = {0x1F, 0x3F, 0x7F};
 
 static void xdot_textspan(GVJ_t * job, pointf p, textspan_t * span)
 {
     emit_state_t emit_state = job->obj->emit_state;
     unsigned flags;
-    char buf[BUFSIZ];
     int j;
     
     agxbput(xbufs[emit_state], "F ");
-    xdot_fmt_num(buf, sizeof(buf), span->font->size);
-    agxbput(xbufs[emit_state], buf);
+    xdot_fmt_num(xbufs[emit_state], span->font->size);
     xdot_str (job, "", span->font->name);
     xdot_pencolor(job);
 
@@ -589,7 +500,8 @@ static void xdot_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	flags = span->font->flags;
     else
 	flags = 0;
-    if (xd->version >= 15) {
+    const size_t flag_masks_size = sizeof(flag_masks) / sizeof(flag_masks[0]);
+    if (xd->version >= 15 && (size_t)xd->version - 15 < flag_masks_size) {
 	unsigned int mask = flag_masks[xd->version-15];
 	unsigned int bits = flags & mask;
 	if (textflags[emit_state] != bits) {
@@ -602,18 +514,16 @@ static void xdot_textspan(GVJ_t * job, pointf p, textspan_t * span)
     agxbput(xbufs[emit_state], "T ");
     xdot_point(xbufs[emit_state], p);
     agxbprint(xbufs[emit_state], "%d ", j);
-    xdot_fmt_num(buf, sizeof(buf), span->size.x);
-    agxbput(xbufs[emit_state], buf);
+    xdot_fmt_num(xbufs[emit_state], span->size.x);
     xdot_str (job, "", span->str);
 }
 
 static void xdot_color_stop (agxbuf* xb, float v, gvcolor_t* clr)
 {
-    char buf[BUFSIZ];
-
-    snprintf(buf, sizeof(buf), "%.03f", v);
-    xdot_trim_zeros (buf, 1);
-    xdot_str_xbuf (xb, buf, color2str (clr->u.rgba));
+  agxbprint(xb, "%.03f", v);
+  agxbuf_trim_zeros(xb);
+  agxbputc(xb, ' ');
+  xdot_str_xbuf(xb, "", color2str (clr->u.rgba));
 }
 
 static void xdot_gradient_fillcolor (GVJ_t* job, int filled, pointf* A, int n)
@@ -680,8 +590,6 @@ static void xdot_ellipse(GVJ_t * job, pointf * A, int filled)
 {
     emit_state_t emit_state = job->obj->emit_state;
 
-    char buf[BUFSIZ];
-
     xdot_style (job);
     xdot_pencolor (job);
     if (filled) {
@@ -695,10 +603,8 @@ static void xdot_ellipse(GVJ_t * job, pointf * A, int filled)
     else
         agxbput(xbufs[emit_state], "e ");
     xdot_point(xbufs[emit_state], A[0]);
-    xdot_fmt_num(buf, sizeof(buf), A[1].x - A[0].x);
-    agxbput(xbufs[emit_state], buf);
-    xdot_fmt_num(buf, sizeof(buf), A[1].y - A[0].y);
-    agxbput(xbufs[emit_state], buf);
+    xdot_fmt_num(xbufs[emit_state], A[1].x - A[0].x);
+    xdot_fmt_num(xbufs[emit_state], A[1].y - A[0].y);
 }
 
 static void xdot_bezier(GVJ_t *job, pointf *A, int n, int filled) {
@@ -744,14 +650,11 @@ void core_loadimage_xdot(GVJ_t * job, usershape_t *us, boxf b, bool filled)
     (void)filled;
 
     emit_state_t emit_state = job->obj->emit_state;
-    char buf[BUFSIZ];
     
     agxbput(xbufs[emit_state], "I ");
     xdot_point(xbufs[emit_state], b.LL);
-    xdot_fmt_num(buf, sizeof(buf), b.UR.x - b.LL.x);
-    agxbput(xbufs[emit_state], buf);
-    xdot_fmt_num(buf, sizeof(buf), b.UR.y - b.LL.y);
-    agxbput(xbufs[emit_state], buf);
+    xdot_fmt_num(xbufs[emit_state], b.UR.x - b.LL.x);
+    xdot_fmt_num(xbufs[emit_state], b.UR.y - b.LL.y);
     xdot_str (job, "", us->name);
 }
 
@@ -807,13 +710,8 @@ gvrender_engine_t xdot_engine = {
     xdot_end_node,
     0,				/* xdot_begin_edge */
     xdot_end_edge,
-#ifdef NEW_XDOT
-    xdot_begin_anchor,
-    xdot_end_anchor,
-#else
     0,                          /* xdot_begin_anchor */
     0,                          /* xdot_end_anchor */
-#endif
     0,				/* xdot_begin_label */
     0,				/* xdot_end_label */
     xdot_textspan,
