@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,10 +5,12 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include "gvplugin.h"
+#include "config.h"
+
+#include <gvc/gvplugin.h>
 
 extern gvplugin_installed_t gvlayout_neato_types[];
 
@@ -20,10 +19,10 @@ static gvplugin_api_t apis[] = {
     {(api_t)0, 0},
 };
 
-#ifdef _WIN32
-#   define GVPLUGIN_NEATO_LAYOUT_API __declspec(dllexport)
+#ifdef GVDLL
+#define GVPLUGIN_NEATO_LAYOUT_API __declspec(dllexport)
 #else
-#   define GVPLUGIN_NEATO_LAYOUT_API
+#define GVPLUGIN_NEATO_LAYOUT_API
 #endif
 
 GVPLUGIN_NEATO_LAYOUT_API gvplugin_library_t gvplugin_neato_layout_LTX_library = { "neato_layout", apis };
